package model;

public class Crop{

	//types of crops considered. Types are treated
	//as equal in all aspects (attributes in this class,
	//labour need, distances/thresholds).
	public static final int TRADITIONALCEREALS=0;
	public static final int HIGHYIELDCEREALS = 1;
	public static final int SUNFLOWER = 2;
	public static final int SUGARBEET =3;
	public static final int POTATOES = 4;
	public static final int VINEYARDS = 5;
	public static final int OLIVES = 6;
	public static final int MELONS = 7;
	public static final int PAPRICA = 8;
	public static final int GARLIC = 9;
	public static final int FALLOW = 10;
	
	public static final int NR_CROPS = 11;
	
	//crop classes used e.g. to compute threshold of changing crops
	public static final int COP = 0;
	public static final int OFC = 1;
	public static final int HORTICULTURE = 2;
	public static final int TREECROP = 3;
	
	
	/**Classifies crops:
	 * 	0: COP (traditional cereals, HighYieldCereals, Sunflower, Fallow
	 *  1: OFC (Other field crops: sugar beet, potatoes
	 *  2: Horticulture (Garlic, Paprica)
	 *  3: Tree crops
	 */	
	public static int cropClassFor(Crop c){
		
		return cropClassFor(c.type);
	}
	
	public static int cropClassFor(int cropType){
		switch(cropType){
			case Crop.TRADITIONALCEREALS:
			case Crop.HIGHYIELDCEREALS:
			case Crop.SUNFLOWER:
			case Crop.FALLOW:return COP;
			case Crop.SUGARBEET:
			case Crop.POTATOES: return OFC;
			case Crop.GARLIC:
			case Crop.MELONS:
			case Crop.PAPRICA:  return HORTICULTURE;
			case Crop.VINEYARDS: 
			case Crop.OLIVES: return TREECROP;
		}	
		return -1;
	}
	
	protected static String name[] = {"tradCereals","highYieldCereals","sunflower",
		"sugarbeet","potatoes","vineyards","olives","melons","paprica","garlic","fallow"
	};
	protected static double yieldRainfed[] = {21.0,21.0,8.0,0.0,0.0,50.0,17.0,0.0,0.0,0.0,0.0};
	protected static double yieldIrrigated[] = {42.0,100.0,18.0,770.0,280.0,100.0,24.0,260.0,215.0,74.0,0.0};
	protected static double price[] = {14.0,15.0,20.0,5.0,21.0,18.0,36.0,27.0,40.0,100.0,0.0};
	//		protected static double price[] = {13.0,14.0,20.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0};
	protected static double waterNeed[] = {2800.0,5200.0,3000.0,6050.0,4900.0, 2350.0,2050.0,4550.0,5900.0,5130.0,0.0};
	protected static double variableCostsExtensive[] = {130.0,750.0,70.0,10000.0,10000.0,200.0,100.0,10000.0,10000.0,10000.0,0.0};
	protected static double variableCostsIntensive[] = {180.0,750.0,260.0,1500.0,2500.0,250.0,150.0,1000.0,1800.0,1300.0,0.0};
	protected static double relStddevRainfed[] = {0.17,0.0,0.22,0.0,0.0,0.25,0.27,0.0,0.0,0.0,0.0};
	protected static double relStddevIrrigated[] = {0.06,0.06,0.13,0.09,1.5,0.13,0.25,0.15,0.13,0.11,0.0};				
	
	public static String getName(int type){
		return name[type];
	}
	public static double getYieldRainfed(int type){// 100kg/ha
		return yieldRainfed[type];
	}
	public static double getYieldIrrigated(int type){// 100kg/ha
		return yieldIrrigated[type];
	}
	public static double getPrice(int type){//  Euro/100kg
		return price[type];
	}
	public static double getWaterNeed(int type){//  m^3/ha/year
		return waterNeed[type];
	}
	public static double getVariableCosts(int type, boolean intensive){//Euro/ha
		if(intensive){
			return variableCostsIntensive[type];
		}
		return variableCostsExtensive[type];
	}
	public static double getRelStddev(int type, boolean irrigated){//stddv of gross margin generated on one ha
		if(irrigated){
			return relStddevIrrigated[type];
		}
			
		return relStddevRainfed[type];
	}	
	
	public static void setName(int type, String name){
		Crop.name[type] = name;
	}
	public static void setYieldRainfed(int type, double yieldRainfed){
		Crop.yieldRainfed[type] = yieldRainfed;
	}	
	public static void setYieldIrrigated(int type, double yieldIrrigated){
		Crop.yieldIrrigated[type] = yieldIrrigated;
	}
	public static void setPrice(int type, double price){
		Crop.price[type] = price;
	}
	public static void setWaterNeed(int type, double waterNeed){
		Crop.waterNeed[type] = waterNeed;
	}
	public static void setVariableCosts(int type, boolean intensive, double directCosts){
		if(intensive){
			Crop.variableCostsIntensive[type] = directCosts;
		}
		else{
			Crop.variableCostsExtensive[type]=directCosts;
		}
		
	}
	public static void setRelStddev(int type, boolean irrigated, double stddev){
		if(irrigated){
			Crop.relStddevIrrigated[type] = stddev;
		}
		else{
			Crop.relStddevRainfed[type] = stddev;
		}
	}
	
	protected int type;	//type of this crop

	public Crop(int type){
		this.type = type;
	}
		
	public double labourNeedPerHa(double size, boolean irrigated){ 	// AWU/ha/year
	
		return Crop.labourNeedPerHa(type, size, irrigated);
	}
	
	/**Returns true if this.type = c.type.*/
	public boolean equals(Crop c){
		return this.type == c.type;
	}
	
	/**Labour need per ha depends on the crop type, on whether the crop is 
	 * irrigated or not (higher yields-> more labour) and on the size of the 
	 * land-use (the bigger the land-use, the more efficient it can be worked on -> less labour per ha). 
	 * The implementation uses a look-up table and linear interpolation
	 * between the defined points.*/
	public static double labourNeedPerHa(int type, double size, boolean irrigated){
				
		
		//size classes, average values of intervalls for which Eurostat data is given 
		double refSizes[] = {1,3.5,7.5,15,25,40,75,150};
		//look-up table. Unit: AWU\100ha
		//organization of table: labour[Crop.type][i][s]
		double labour [][][] = {
			{	{10.0, 7.0,3.5,2.5,2.0,1.5,1.0,0.7},		//trad. cereals rainfed
				{12.0,8.0,4.0,3.0,2.5,2.0,1.5,0.9}},	//trad. cereals irrigated
			{	{-1.0,-1.0,-1.0,-1.0,-1.0,-1.0,-1.0,-1.0},	//HY cereals rainfed
				{15.0,10.0,5.0,4.0,3.0,2.5,2.0,1.2}},       //HY cereals irrigated
			{	{10.0, 7.0,3.5,2.5,2.0,1.5,1.0,0.7},			//sunflower rainfed
				{12.0,8.0,4.0,3.0,2.5,2.0,1.5,0.9}},		//sunflower irrigated
			{	{-1.0,-1.0,-1.0,-1.0,-1.0,-1.0,-1.0,-1.0},	//sugarbeet rainfed
				{30.0,20.0,14.0,11.0,8.0,7.0,6.0,5.0}},		//sugarbeet irrigated
			{	{-1.0,-1.0,-1.0,-1.0,-1.0,-1.0,-1.0,-1.0},	//potatoes rainfed
				{50.0,23.0,17.0,9.0,7.5,6.0,6.0,3.0}},		//potatoes irrigated
			{	{12.0,7.0,6.0,5.0,4.0,3.5,3.0,3.0},			//vineyards rainfed
				{17.0,9.0,8.0,7.0,5.5,5.0,4.0,4.0}},		//vineyards irrigated
			{	{8.0,7.0,6.0,5.0,4.5,4.0,3.5,3.0},			//olives rainfed
				{10.0,9.0,8.0,7.0,6.0,5.0,4.5,4.0}},		//olives irrigated
			{	{-1.0,-1.0,-1.0,-1.0,-1.0,-1.0,-1.0,-1.0},	//melons rainfed
				{50.0,25.0,17.0,12.0,10.0,8.0,7.0,6.0}},//melons irrigated
			{	{-1.0,-1.0,-1.0,-1.0,-1.0,-1.0,-1.0,-1.0},	//paprica rainfed
				{70.0,35.0,27.0,24.0,21.0,20.0,20.0,20.0}},//paprica irrigated
			{	{-1.0,-1.0,-1.0,-1.0,-1.0,-1.0,-1.0,-1.0},	//garlic rainfed
				{45.0,22.0,15.0,12.0,11.0,10.5,10.0,9.5}},//garlic irrigated
			{	{0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0},			//fallow rainfed
				{0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0}}			//fallow irrigated
		};
		
		//i will be used to select 2. grade in table: [][i][] 
		int i = irrigated? 1:0;
		//l will store value
		double l;		
		
		if(size <= refSizes[0]){//special case, no linear interpolation but fixed value
			l  = labour[type][i][0];	
		}
		else{
			if(size >= refSizes[refSizes.length-1]){//special case assume no further economies of scale
				l  = labour[type][i][refSizes.length-1];	
			}
			else{
				int s = 1;	//size>refSizes[0] is given, therefore s=0 need not be tested
				while(size>refSizes[s]){	//size < refSizes[refSizes.length-1] is given
					s++;
				}
				l = labour[type][i][s-1]+ (size-refSizes[s-1])/(refSizes[s]-refSizes[s-1])*(labour[type][i][s]-labour[type][i][s-1]);
			}
		}
		if(l<0.0){
			throw
				new RuntimeException("Crop:labour look-up table gives negative value!");	
		}
		l = l/100.0;				//normalize to AWU\ha		
		return l;
	}
	
	public String getName(){
		return getName(this.type);
	}
	
	public int getType(){
		return type;
	}
	
	public double getYieldRainfed(){
		return getYieldRainfed(this.type); // 100kg/ha
	}
	public double getYieldIrrigated(){
		return getYieldIrrigated(this.type);// 100kg/ha
	}
	public double getPrice(){
		return getPrice(this.type); 		//  Euro/100kg
	}
	public double getWaterNeed(){
		return getWaterNeed(this.type);	//  m^3/ha/year
	}
	public double getVariableCosts(boolean irrigated){
		return getVariableCosts(this.type, irrigated);	//Euro/ha	
	}
	public double getRelStddev(boolean irrigated){
		
		return getRelStddev(this.type, irrigated); //stddv of gross margin generated on one ha
	}
}
