package model;


import model.decisionMaking.*;

/**
*/
public class Farmer { 
	
	/**Define order of evaluation criteria (priorities) in array*/
/*	public static final int GROSSMARGIN = 0;			
	public static final int LABOUR_MAX = 3;		
	public static final int LABOUR_SHAPE = 4;	
	public static final int RISK = 2;		
	public static final int LEGAL = 1;		
*/
	//*Farm sizes*/
	public static final int VERY_SMALL = 0;
	public static final int SMALL = 1;
	public static final int MEDIUM1 = 2;
	public static final int MEDIUM2 = 3;
	public static final int BIG = 4;
	//Define here farm sizes to be used by Farmer types
	public static double [] FARM_SIZES = {4.0,8.0,32.0,70.0,150.0};
	
	public static final int PART_TIME = 0;
	public static final int FAMILY = 1;
	public static final int BUSINESS = 2;
	
	public static double[] FAMILY_LABOUR = {0.4,1.0,0.4};
	
	//learning rate = 1/lTime, i.e. the higher lTime, the slower farmers learn
	public static double lTime = 3.0;
	
	public class StockOfIrrigationTech{
		
		double sprinklerArea;
		double dripArea;
		double rainfedArea;
		double waterCapacity;
		
		public double getSprinklerArea(){
			return sprinklerArea;
		}
		public double getDripArea(){
			return dripArea;
		}
		public double getRainfedArea(){
			return sizeOfFarm;
		}
		public double getWaterCapacity(){
			return waterCapacity;
		}
		
		public void setSprinklerArea(double a){
			sprinklerArea = Math.max(0,Math.min(a,sizeOfFarm));
		}
		public void setDripArea(double a){
			dripArea = Math.max(0,Math.min(a,sizeOfFarm));
		}
		public void setRainfedArea(double a){
			rainfedArea = Math.max(0,Math.min(a,sizeOfFarm));
		}
		public void setWaterCapacity(double c){
			waterCapacity = Math.max(0,Math.min(c,sizeOfFarm*GlobalVariables.maxWaterUse));
		}
	}
	
	
	/**Creates and returns a ChoiceProcess Object which defines
	 * how a farmer of type "type" makes decisions.
	 * @param type: the farmer's type
	 * @return
	 */
	protected static ChoiceProcess choiceProcessForType(int type){
		
		//implement heterogeneity of farmer types in terms of decision algorithm here
		CollectOptions cO = null;
		if(GlobalVariables.DIFFUSION){
			cO = new CollectOptions_DiffusionNrPeople();
		}
		GenerateLandUsePattern gP = null;
		RuleApplication rA = null;
		CompareLandUsePattern cP = null;
		RefinePattern rP = null;
		ChoiceProcess cp = new ChoiceProcess(cO,gP,rA,cP,rP, type);
		return cp;
	}

	public double sizeOfFarm;
	
	public double familyLabour;
	
	public double cropSkills[] = new double[11];
	public double techSkills[] = new double[4];
	public StockOfIrrigationTech irrigationTech;

	public EvaluatedLandUsePattern elp;	
	public ChoiceProcess choiceProcess;
	
	public int type;
	public int sizeClass;
	
	/*	//Can be used to define own types
	public Farmer(int type, ChoiceProcess choiceP, double sizeOfFarm,double profit_weight, double labourShapeFactor, double risk_weight, double stayLegal_weight, 
			double distanceSkillsShapeFactor, double distanceCapitalShapeFactor, double maxLabourLoadFactor){
		
		this.choiceProcess = choiceP;
		this.sizeOfFarm = sizeOfFarm;
		utilityFactors = new double[5];
		this.utilityFactors[GROSSMARGIN] = profit_weight;
		this.utilityFactors[LEGAL] = stayLegal_weight;
		this.utilityFactors[RISK] = risk_weight;
		this.utilityFactors[LABOUR_MAX] = maxLabourLoadFactor;
		this.utilityFactors[LABOUR_SHAPE] = labourShapeFactor;
		
		distanceFactors = new double[3];
		this.distanceFactors[Distances.SKILLS]=distanceSkillsShapeFactor;
		this.distanceFactors[Distances.CAPITAL]=distanceCapitalShapeFactor;
	this.labourForce = maxLabourLoadFactor * 1.0; 		//labour load set per default to 1.0*maxLabourLoadFactor, need to be adjusted when setting land-use pattern
		this.type = type;
		
		//default knowlegde: fallow and rainfed
		knownCrops[Crop.FALLOW] = true;
		irrigationTech = new StockOfIrrigationTech();
//		ownedTechAreas[IrrigationTechnology.RAINFED] = sizeOfFarm;
	}
*/		
	//Specifies a farmer according to pre-defined farmer types and size classes
	public Farmer(int type, int sizeClass){
		//farmer types are defined starting with type1, therefore use type-1 to refer to variable values stored in arrays
		this.choiceProcess = choiceProcessForType(type);
		this.sizeOfFarm = FARM_SIZES[sizeClass];
		this.familyLabour = FAMILY_LABOUR[type];
		this.type = type;
		this.sizeClass = sizeClass;
		cropSkills[Crop.FALLOW] = 1.0;
		techSkills[IrrigationTechnology.RAINFED]=1.0;
		irrigationTech = new StockOfIrrigationTech();
	}
	
	/**Wraps lp in an EvaluatedLandUsePattern elp and calls setLandUsePattern(EvaluatedLandUsePattern elp). 
	 * elp.pattern is set to lp, all other variables are not specified (i.e. they are 0 or null). 
	 */
	public void setInitialLandUsePattern(LandUsePattern lp){
		
		EvaluatedLandUsePattern dummy = new EvaluatedLandUsePattern();
		dummy.pattern = lp;
		setLandUsePattern(dummy);
		
	}
	
	/**The landUsePattern of this farmer is set to elp. The method decreases / increases
	 * uses of Options in this.landUsePattern and in lp (this is necessary to ensure
	 * that o.getUses() yields correct information. The farmer learns.
	 * @param lp
	 */
	public void setLandUsePattern(EvaluatedLandUsePattern elp){
		
		Option o;
		LandUsePattern lp; 
		
		//something has changed
		if(this.elp != elp){
/*			//reduce uses of current landuse (may be ommitted to change
			//meaning of diffusion algorithm and include common knowledge
			//on history?!
			if(this.elp != null){
				lp = this.elp.pattern;
				for(int i=0; i<lp.length(); i++){
					o = lp.get(i).option;
					o.setUses(o.getUses()-1);
				}
			}
*/
			if(elp!=null){
				lp = elp.pattern;
				//increase uses of new options in new elp
				for(int i=0; i<lp.length(); i++){
					o = lp.get(i).option;
					if(this.elp== null || this.elp.pattern.indexOf(o)<0){	//an option which is not included in current landusePattern
						o.setUses(o.getUses()+1);
					}
				}
				//update tech stock (only increase stock)
				irrigationTech.setDripArea(Math.max(irrigationTech.getDripArea(),lp.irrigationTechArea(IrrigationTechnology.DRIP)));
				irrigationTech.setSprinklerArea(Math.max(irrigationTech.getSprinklerArea(),lp.irrigationTechArea(IrrigationTechnology.SPRINKLER)));
				irrigationTech.setWaterCapacity(Math.max(irrigationTech.getWaterCapacity(),lp.waterUsed()));		
			}		
		}
		
		//learn about options (every year, not only if something has changed)
		if(elp!= null){
			lp = elp.pattern;
			for(int i=0; i<lp.length(); i++){
				o = lp.get(i).option;
				Farming fa = (Farming) o;
		
				//improve crop skills
				double cs = cropSkills[fa.getCrop().getType()];
				if(cs == 0.0){
					cs = 1.0/lTime;
				}
				else{
					cs = cs+(1.0-cs)/lTime;
				}
				cropSkills[fa.getCrop().getType()] = cs;
		
				//improve tech skills
				double ts = techSkills[fa.getIrrigationTechnology().getType()];
				if(ts == 0.0){
					ts = 1.0/lTime;
				}
				else{
					ts = ts+(1.0-ts)/lTime;
				}
				techSkills[fa.getIrrigationTechnology().getType()] = ts;
			}
		}
		this.elp = elp;
	}
	
	public LandUsePattern landUsePattern(){
		return elp.pattern;
	}
	
	public EvaluatedLandUsePattern evaluatedLandUsePattern(){
		return elp;
	}
	
	
	public void setFamilyLabour(double l){
		this.familyLabour = l;	
	}
	
	/**Costs of labour to do lp; does not count family labour*/
	public double labourCosts(LandUsePattern lp){
		
		return Math.max(0.0, lp.labour()-this.familyLabour)*GlobalVariables.labourCosts;
	}

	
	public double cropSkill(int cropType){
		return cropSkills[cropType];
	}
	
	public boolean knowsCrop(int cropType){
		return cropSkills[cropType] > 0.0;
	}
	
	public double techSkill(int techType){
		return techSkills[techType];
	}
	
	public boolean knowsTech(int techType){

		return techSkills[techType]>0.0;

	}
	
	/**In each step a farmer reconsiders his landUsePattern as specified in f.choiceProcess.*/
	public void action(){
		EvaluatedLandUsePattern newPattern= choiceProcess.designLandPattern(this);
		//Clone pattern to conserve current attributes of Options
		setLandUsePattern(newPattern);
	}
}
