package model;

import uchicago.src.sim.engine.BasicAction;
import uchicago.src.sim.engine.Schedule;
import uchicago.src.sim.engine.SimModelImpl;
import uchicago.src.sim.util.Random;
import uchicago.src.sim.util.SimUtilities;
import uchicago.src.sim.analysis.plot.OpenGraph;

import java.util.ArrayList;
import java.util.Iterator;

import model.decisionMaking.*;
import visualization.*;

import model.rules.*;

/**The main class of this model which sets the scene and controls the program flow.
 * 
 * This Class has been edited by Marvin Nebel to include Satisficing
 * Several small changes were necessary, they are marked with "//CHANGE:"
 */


public class UGBModelDisplay extends SimModelImpl implements YearCount {
	
	public int year;
	public int lastYear = 2010;
	
	protected Schedule schedule;
	protected boolean writeToFile = true;

	//These lists contain all elements of this model
	protected ArrayList<Farmer> farmers;
	protected ArrayList<Option> options;
	protected ArrayList<Rule> rules;
	
	
	//visualization
	protected ArrayList<ObserveFarmerChart> farmerCharts;
	protected ArrayList<EtcChart> etcCharts;
	
	
	/////////////////
	////// setup
	/////////////
	
	public void setup() {
		
		//set output path for files
		GlobalVariables.setSubDir("_"+this.getController().getRunCount()+"/");
		(new java.io.File(GlobalVariables.getOutputDir())).mkdir();
		
		// create a schedule with an interval of one.
		schedule = new Schedule(1);
		//visualization
		setupDisplay();
		
		
	}
	
	protected void setupDisplay(){
		
		/////////////////////////
		//Charts that show farmers' land use
		
		//This array specifies which farmer should be observed in detail 
		//(a SingleFarmerLandUsePatternChart is created for each farmer in this array)
		int showDetailsOfFarmerNr[] ={5,/*50,95, 110,150,195,*/225,/*270,290,325,370,390,*/450/*,475,495*/};
		
			
		//Dispose charts from last model run
		if(farmerCharts != null){
			Iterator<ObserveFarmerChart> it = farmerCharts.iterator();
			while( it.hasNext()){
				it.next().dispose();
			}
		}
		
		farmerCharts = new ArrayList<ObserveFarmerChart>(); 
		ObserveFarmerChart chart;
		
		//////create Single Farmer Landuse Charts
		for(int i=0; i<showDetailsOfFarmerNr.length; i++){
			int nr = showDetailsOfFarmerNr[i];
			chart =  new SingleFarmerLandUsePatternChart("Land use Farmer "+nr, this,nr);
			farmerCharts.add(chart);
//			chart = new SingleFarmerWaterChart("Water use Farmer "+nr, this, nr);
//			farmerCharts.add(chart);
//			chart =  new SingleFarmerSubsidiesChart("Subsidies "+nr, this,nr);
//			farmerCharts.add(chart);
			
			//CHANGE: THIS CHART NEEDS TO BE TAKEN OUT FOR THE SATISFICING VERSION
			if (! (GlobalVariables.SATISFICING_PER_TYPE[0] ||GlobalVariables.SATISFICING_PER_TYPE[1] ||GlobalVariables.SATISFICING_PER_TYPE[2])){
				chart =  new SingleFarmerScoreChart("Scores Farmer "+nr, this,nr);
				farmerCharts.add(chart);
			}
			else if ((GlobalVariables.SATISFICING_PER_TYPE[0] && GlobalVariables.SATISFICING_PER_TYPE[1] && GlobalVariables.SATISFICING_PER_TYPE[2])){
				chart = new SingleFarmerThresholdOutputChart("Threshold Output Farmer " + nr, this, nr);
				farmerCharts.add(chart);
			}
			
		}
		//Chart for Satisficing Version Average Output Threshold Chart
		chart = new AverageThresholdOutputChart("Average threshold and output", this);
		farmerCharts.add(chart);
		//Chart for total land use. Needs to scale up land-use of different types accordingly.
		chart = new TotalLandUsePatternChart("Total landUse", this);
		farmerCharts.add(chart);
		chart = new TotalCropAreaChart("Total crop area", this);
		farmerCharts.add(chart);
		
/*		chart = new CropCategoryAreaChart("Area of crop categories", this);
		farmerCharts.add(chart);
*/
		
/**/			
		chart = new FarmerSizeClassLandUsePatternChart("Very small farmer (4ha)", this,Farmer.VERY_SMALL);
		farmerCharts.add(chart);
		
		chart = new FarmerSizeClassLandUsePatternChart("Small farmer (8 ha)", this,Farmer.SMALL);
		farmerCharts.add(chart);
		
		chart = new FarmerSizeClassLandUsePatternChart("Medium1 farmer (32 ha)", this,Farmer.MEDIUM1);
		farmerCharts.add(chart);
		
		chart = new FarmerSizeClassLandUsePatternChart("Medium2 farmer (70 ha)", this,Farmer.MEDIUM2);
		farmerCharts.add(chart);
		
		chart = new FarmerSizeClassLandUsePatternChart("Big farmer (150 ha)", this,Farmer.BIG);
		farmerCharts.add(chart);
		
		chart = new FarmerTypeLandUsePatternChart("Business farms", this,Farmer.BUSINESS);
		farmerCharts.add(chart);
		
		chart = new FarmerTypeLandUsePatternChart("Family farms", this,Farmer.FAMILY);
		farmerCharts.add(chart);
		
		chart = new FarmerTypeLandUsePatternChart("Part-time farms", this,Farmer.PART_TIME);
		farmerCharts.add(chart);
		
/**/		
		
/*		
		chart = new FarmerSizeClassCropChart("Small farmer (8ha)", this,7.5,8.5);
		farmerCharts.add(chart);
		
		chart = new FarmerSizeClassCropChart("Medium farmer (40ha)", this,39.5,40.5);
		farmerCharts.add(chart);
		
		chart = new FarmerSizeClassCropChart("Big farmer", this,100.0,500.0);
		farmerCharts.add(chart);	
//*/		
//*		
		chart = new AepAreaChart("Area under AEP", this);
		farmerCharts.add(chart);
		
		chart = new CropCategoryAEPAreaChart("Crop categories under AEP", this);
		farmerCharts.add(chart);
		
		chart = new AepPaymentsChart("Payments within AEP", this);
		farmerCharts.add(chart);
		
		chart = new CapCompPaymentsChart("Direct payments (CAP)", this);
		farmerCharts.add(chart);
//*/
//*		
		chart = new IrrigatedAreaPerTechChart("Irrigation Technology used", this);
		farmerCharts.add(chart);
		
		chart = new IrrigatedCropAreaChart("Irrigated crops", this);
		farmerCharts.add(chart);
		
		chart = new WaterExtractionsChart("Water extracted from aquifer", this);
		farmerCharts.add(chart);
//*/
//*		
		chart = new IllegalFarmersChart("Illegal Farmers", this);
		farmerCharts.add(chart);
//*/		
/*		chart = new LabourInputChart("Labour input", this);
		farmerCharts.add(chart);
//*/		
		
		
		/////////////////////////////////////////////////
		//other charts, e.g. total number of uses, water use etc.		
		
		//Dispose charts from last run
		if(etcCharts != null){
			Iterator<EtcChart> it = etcCharts.iterator();
			while( it.hasNext()){
				it.next().dispose();
			}
		}	
		
		etcCharts = new ArrayList<EtcChart>();
		
//*		
		EtcChart etc = new TotalUsesChart("Total Uses of Options", this);
		etcCharts.add(etc);
//*/	
	}
	

	public void begin() {
		
		clearData();	//clear all global data which was potentially changed during last run
		Random.createUniform();			//create uniform random distribution
		Random.createNormal(1.0,1.0); //create normal random distribution with mean = 1.0, sd=1.0 
		year = 1960;			//first year of simulation
		buildModel();
		buildSchedule();
		buildDisplay();

	}

	
	
	///////////////////////////
	/// Model /////////////
	//////////////////////////
	
	
	protected void buildModel(){
		
		//Same for all farmers
		options = createOptions();
		rules = createRules();
		createFarmers();
		
		GlobalVariables.setUp(farmers, options);
	}
	
	public ArrayList<Option> createOptions(){
		
		ArrayList<Option> allOptions = new ArrayList<Option>();
		
		//Farming
		//create all Crops
		Crop[] allCrops = new Crop[11];
													//name, type, yieldRainfed, yieldIrrigated, price, waterNeed, directCosts, stddevRainfed, stddevIrrigated
		allCrops[Crop.TRADITIONALCEREALS] = new Crop(Crop.TRADITIONALCEREALS);
		allCrops[Crop.HIGHYIELDCEREALS] = new Crop(Crop.HIGHYIELDCEREALS);
		allCrops[Crop.SUNFLOWER] = new Crop(Crop.SUNFLOWER);
		allCrops[Crop.SUGARBEET] = new Crop(Crop.SUGARBEET);
//		allCrops[Crop.POTATOES] = new Crop(Crop.POTATOES);
		allCrops[Crop.VINEYARDS] = new Crop(Crop.VINEYARDS);
		allCrops[Crop.OLIVES] = new Crop(Crop.OLIVES);
		allCrops[Crop.MELONS] = new Crop(Crop.MELONS);
		allCrops[Crop.PAPRICA] = new Crop(Crop.PAPRICA);
		allCrops[Crop.GARLIC] = new Crop(Crop.GARLIC);
		allCrops[Crop.FALLOW] = new Crop(Crop.FALLOW);

		//create all IrrigationTechnologies
		IrrigationTechnology[] allTechs;
		//*with irrigation
		if(GlobalVariables.IRRIGATION){
			allTechs= new IrrigationTechnology[4];																//String name, int type, double waterEfficiency
			allTechs[IrrigationTechnology.RAINFED] = new IrrigationTechnology("rainfed",IrrigationTechnology.RAINFED, 1.0);
			allTechs[IrrigationTechnology.FLOOD] = new IrrigationTechnology("flood",IrrigationTechnology.FLOOD ,0.5);
			allTechs[IrrigationTechnology.SPRINKLER] = new IrrigationTechnology("sprinkler",IrrigationTechnology.SPRINKLER, 0.75);
			allTechs[IrrigationTechnology.DRIP] = new IrrigationTechnology("drip",IrrigationTechnology.DRIP, 0.9);
		}
		//*/
		else{
			//*/ without irrigation 		
			allTechs= new IrrigationTechnology[1];	
			allTechs[IrrigationTechnology.RAINFED] = new IrrigationTechnology("rainfed",IrrigationTechnology.RAINFED, 1.0);
			}
		//*/		
		//create all compatible combinations of Crop and IrrigationTechnology
		Farming f;
		for(int c=0; c<allCrops.length; c++){
			if(allCrops[c]!=null){
				for(int t=0;t<allTechs.length; t++){
					if(allTechs[t]!=null){
						if(Farming.compatibility(allCrops[c],allTechs[t])){
							f = new Farming(allCrops[c],allTechs[t]);
							allOptions.add(f);
						}
					}
				}
			}
		}
//*debug*/System.err.println(allOptions);
		
		return allOptions;
	}

	public ArrayList<Rule> createRules(){
		
		rules = new ArrayList<Rule>();
		
		//CAP_Prices changes prices of COP products according to CAP intervention prices
		CapPrices capPrices = new CapPrices();
		capPrices.setYearInfo(this); //is increased in UGBModel.prepareStep() calling CAP_Prices.prepareNextStep()
		rules.add(capPrices);
		
		//Compensatory payments, i.e. direct payments 
		CapCompPayments capSubsidies = new CapCompPayments();
		capSubsidies.setYearInfo(this);
		rules.add(capSubsidies);
		
		
		VineLaw vl = new VineLaw();
		vl.setYearInfo(this);
		rules.add(vl);
		
		
		//PumpingQuotas
		PumpingQuotas pq = new PumpingQuotas();
		pq.setYearInfo(this);
		rules.add(pq);
		
		//Agro-environmental programme
		AEP aep = new AEP(pq);
		aep.setYearInfo(this);
		rules.add(aep);
		
		return rules;
	}
	
	//The following methods define initial farm types and 
	//are used by createFarmers()
	protected LandUsePattern tradCereals(double sizeOfFarm){
		LandUsePattern lp = new LandUsePattern();
		LandUse l = new LandUse();
		l.option = Farming.findFarmingOption(options, Crop.TRADITIONALCEREALS, IrrigationTechnology.RAINFED);
		l.size = sizeOfFarm*0.7;	
		lp.add(l);
		l = new LandUse();
		l.option = Farming.findFarmingOption(options, Crop.FALLOW, IrrigationTechnology.RAINFED);
		l.size = sizeOfFarm*0.3;	
		lp.add(l);
		return lp;
	}
	protected LandUsePattern vineyards(double sizeOfFarm){
		LandUsePattern lp = new LandUsePattern();
		LandUse l = new LandUse();
		l.option = Farming.findFarmingOption(options, Crop.VINEYARDS, IrrigationTechnology.RAINFED);
		l.size = sizeOfFarm*0.8;
		lp.add(l);
		l = new LandUse();
		l.option = Farming.findFarmingOption(options, Crop.FALLOW, IrrigationTechnology.RAINFED);
		l.size = sizeOfFarm*0.2;
		lp.add(l);
		return lp;
	}
	protected LandUsePattern olives(double sizeOfFarm){
		LandUsePattern lp = new LandUsePattern();
		LandUse l = new LandUse();
		l.option = Farming.findFarmingOption(options, Crop.OLIVES, IrrigationTechnology.RAINFED);
		l.size = sizeOfFarm*0.8;
		lp.add(l);
		l = new LandUse();
		l.option = Farming.findFarmingOption(options, Crop.FALLOW, IrrigationTechnology.RAINFED);
		l.size = sizeOfFarm*0.2;
		lp.add(l);
		return lp;
	}
	protected LandUsePattern mixed(double sizeOfFarm){
		LandUsePattern lp = new LandUsePattern();
		LandUse l = new LandUse();
		l.option = Farming.findFarmingOption(options, Crop.VINEYARDS, IrrigationTechnology.RAINFED);
		l.size = sizeOfFarm*0.2;
		lp.add(l);
		l = new LandUse();
		l.option = Farming.findFarmingOption(options, Crop.OLIVES, IrrigationTechnology.RAINFED);
		l.size = sizeOfFarm*0.1;
		lp.add(l);
		l = new LandUse();
		l.option = Farming.findFarmingOption(options, Crop.TRADITIONALCEREALS, IrrigationTechnology.RAINFED);
		l.size = sizeOfFarm*0.4;
		lp.add(l);	
		l = new LandUse();
		l.option = Farming.findFarmingOption(options, Crop.FALLOW, IrrigationTechnology.RAINFED);
		l.size = sizeOfFarm*0.3;
		lp.add(l);
		return lp;
	}
	
	//**** Intial crop specialisations*/
	static final int spCOP = 0, spVine=1, spOlive=2, mix=3;
	////////////////////////////////////
	
	
	//CHANGE: Create Farmer_Satisficing if necessary
	protected void createFarmers(int nr, int type,int sizeClass, int spec){
		
		Farmer_Satisficing fs;
		Farmer f;
		switch(spec){ 
			case spCOP: 
				for(int i=0; i<nr; i++){
					if (GlobalVariables.SATISFICING_PER_TYPE[type]){
						fs = new Farmer_Satisficing(type, sizeClass);
						fs.setInitialLandUsePattern(tradCereals(fs.sizeOfFarm));
						farmers.add(fs);
					}
					else {
						f = new Farmer(type, sizeClass);
						f.setInitialLandUsePattern(tradCereals(f.sizeOfFarm));
						farmers.add(f);
					}
				}
				break;
				
			case spVine: 
				for(int i=0; i<nr; i++){
					if (GlobalVariables.SATISFICING_PER_TYPE[type]){
						fs = new Farmer_Satisficing(type, sizeClass);
						fs.setInitialLandUsePattern(vineyards(fs.sizeOfFarm));
						farmers.add(fs);
					}
					else {
						f = new Farmer(type, sizeClass);
						f.setInitialLandUsePattern(vineyards(f.sizeOfFarm));
						farmers.add(f);
					}
				}
				break;
				
			case spOlive: 
				for(int i=0; i<nr; i++){
					if (GlobalVariables.SATISFICING_PER_TYPE[type]){
						fs = new Farmer_Satisficing(type, sizeClass);
						fs.setInitialLandUsePattern(olives(fs.sizeOfFarm));
						farmers.add(fs);
					}
					else {
						f = new Farmer(type, sizeClass);
						f.setInitialLandUsePattern(olives(f.sizeOfFarm));
						farmers.add(f);
					}
				}
				break;
				
			default: 
				for(int i=0; i<nr; i++){
					if (GlobalVariables.SATISFICING_PER_TYPE[type]){
						fs = new Farmer_Satisficing(type, sizeClass);
						fs.setInitialLandUsePattern(mixed(fs.sizeOfFarm));
						farmers.add(fs);
					}
					else{
						f = new Farmer(type, sizeClass);
						f.setInitialLandUsePattern(mixed(f.sizeOfFarm));
						farmers.add(f);
					}
				}
		}
	}
	

	public void createFarmers(){
		farmers = new ArrayList<Farmer>();
		int init = GlobalVariables.init;
		
		///////////////////////////////////
		//sizeClass 1: Very small farmers
		//shares of farmer specialisations for different model initialisations
		
		double[][][] initDistVS=
		{{	//init 1
			{0.0, 0.0, 0.0}, //spCOP
			{0.32,0.35,0.0}, //spVine
			{0.22,0.11,0.0}, //spOlive
			{0.0,0.0,0.0}	   //mix
		},
		{	//init 2
			{0.0, 0.0, 0.0}, //spCOP
			{0.32,0.35,0.0}, //spVine
			{0.22,0.11,0.0}, //spOlive
			{0.0,0.0,0.0}	   //mix
		},
		{	//init 3
			{0.0, 0.0, 0.0}, //spCOP
			{0.32,0.35,0.0}, //spVine
			{0.22,0.11,0.0}, //spOlive
			{0.0,0.0,0.0}	   //mix
		}};
		
		for(int s=0;s<4;s++){	//specialisation
			for(int t=0;t<3; t++){			//type
				createFarmers((int)(GlobalVariables.nrFarmersModelPerSizeClass*initDistVS[init][s][t]),
						t,Farmer.VERY_SMALL,s);
			}
		}
		
		
		///////////////////////////////////
		//sizeClass 2: Small farmers
		double[][][] initDistS=
		{{	//init 1
			{0.0, 0.0, 0.0}, //spCOP
			{0.14,0.46,0.0}, //spVine
			{0.2,0.0,0.0}, //spOlive
			{0.0,0.2,0.0}	   //mix
		},
		{	//init 2
			{0.0, 0.0, 0.0}, //spCOP
			{0.14,0.46,0.0}, //spVine
			{0.2,0.0,0.0}, //spOlive
			{0.0,0.2,0.0}	   //mix
		},
		{	//init 3
			{0.0, 0.0, 0.0}, //spCOP
			{0.14,0.46,0.0}, //spVine
			{0.2,0.0,0.0}, //spOlive
			{0.0,0.2,0.0}	   //mix
		}};
		
		for(int s=0;s<4;s++){	//specialisation
			for(int t=0;t<3; t++){			//type
				createFarmers((int)(GlobalVariables.nrFarmersModelPerSizeClass*initDistS[init][s][t]),
						t,Farmer.SMALL,s);
			}
		}
		
		
		///////////////////////////////////
		//sizeClass 3: Medium1 farmers

		double[][][] initDistM1=
		{{	//init 1
			{0.0, 0.0, 0.0}, //spCOP
			{0.0,0.66,0.17}, //spVine
			{0.0,0.0,0.0}, //spOlive
			{0.0,0.17,0.0}	   //mix
		},
		{	//init 2
			{0.0, 0.0, 0.0}, //spCOP
			{0.0,0.66,0.17}, //spVine
			{0.0,0.0,0.0}, //spOlive
			{0.0,0.17,0.0}	   //mix
		},
		{	//init 3
			{0.0, 0.0, 0.0}, //spCOP
			{0.0,0.66,0.17}, //spVine
			{0.0,0.0,0.0}, //spOlive
			{0.0,0.17,0.0}	   //mix
		}};
		
		for(int s=0;s<4;s++){	//specialisation
			for(int t=0;t<3; t++){			//type
				createFarmers((int)(GlobalVariables.nrFarmersModelPerSizeClass*initDistM1[init][s][t]),
						t,Farmer.MEDIUM1,s);
			}
		}
		
		
		///////////////////////////////////
		//sizeClass 4: Medium2 farmers

		double[][][] initDistM2=
		{{	//init 1
			{0.0, 0.0, 0.43}, //spCOP
			{0.0,0.0,0.43}, //spVine
			{0.0,0.0,0.0}, //spOlive
			{0.0,0.16,0.0}	   //mix
		},
		{	//init 2
			{0.0, 0.0, 0.43}, //spCOP
			{0.0,0.0,0.43}, //spVine
			{0.0,0.0,0.0}, //spOlive
			{0.0,0.16,0.0}	   //mix
		},
		{	//init 3
			{0.0, 0.0, 0.43}, //spCOP
			{0.0,0.0,0.43}, //spVine
			{0.0,0.0,0.0}, //spOlive
			{0.0,0.16,0.0}	   //mix
		}};
		
		for(int s=0;s<4;s++){	//specialisation
			for(int t=0;t<3; t++){			//type
				createFarmers((int)(GlobalVariables.nrFarmersModelPerSizeClass*initDistM2[init][s][t]),
						t,Farmer.MEDIUM2,s);
			}
		}
		
		
		///////////////////////////////////
		//sizeClass 5: Big farmers
		double[][][] initDistB=
		{{	//init 1
			{0.0, 0.0, 0.83}, //spCOP
			{0.0,0.0,0.0}, //spVine
			{0.0,0.0,0.0}, //spOlive
			{0.0,0.0,0.17}	   //mix
		},
		{	//init 2
			{0.0, 0.0, 0.83}, //spCOP
			{0.0,0.0,0.0}, //spVine
			{0.0,0.0,0.0}, //spOlive
			{0.0,0.0,0.17}	   //mix
		},
		{	//init 3
			{0.0, 0.0, 0.83}, //spCOP
			{0.0,0.0,0.0}, //spVine
			{0.0,0.0,0.0}, //spOlive
			{0.0,0.0,0.17}	   //mix
		}};
		
		for(int s=0;s<4;s++){	//specialisation
			for(int t=0;t<3; t++){			//type
				createFarmers((int)(GlobalVariables.nrFarmersModelPerSizeClass*initDistB[init][s][t]),
						t,Farmer.BIG,s);
			}
		}
		
		//provide information to initiate farmers' choice process
		Iterator<Farmer> it = farmers.iterator();
		Farmer f;	
		while(it.hasNext()){
			f = it.next();
			f.choiceProcess.setUp(options,rules,farmers.size());
		
		}

		
	}
	
	
	///////////////////////////
	/// Schedule /////////////
	//////////////////////////
	
	
	protected void buildSchedule(){
		
		 class BasicSchedule extends BasicAction{
			  public void execute() {
				  preStep();
				  step();
				  postStep();

			  }
		  };
		  BasicSchedule runModel = new BasicSchedule();
		  schedule.scheduleActionBeginning(1, runModel);
		  if(writeToFile){
			  schedule.scheduleActionAtEnd(farmerCharts, ObserveFarmerChart.class, "writeToFile");
		  }
	}
	
	protected void preStep() {
		
		//Draw data in preStep to include initial state of model in graphical output.
		Iterator<ObserveFarmerChart> it = farmerCharts.iterator();
		while(it.hasNext()){
			it.next().step();
		};
		
		Iterator<EtcChart> it2 = etcCharts.iterator();
		while(it2.hasNext()){
			it2.next().step();
		};
		
	}
	
	
	protected void step() {
		
		this.year++;
		// Randomize the order of farmers
		  shuffleAgents();
		 //Prepare rules 
		  if(rules != null){
			  Iterator<Rule> it3 = rules.iterator();
			  while(it3.hasNext()){
				  it3.next().prepareNextStep();
			  }
		  }
		
		  /*Call action() of all Farmers.*/
		  // CHANGE: SATISFICING INCLUDED
		  Iterator<Farmer> it = farmers.iterator();
		  Farmer f;
		  while(it.hasNext()){ 
			  f = it.next();
			  
			  // if farmer is satisficing is applied for this farmer type
			  if (GlobalVariables.SATISFICING_PER_TYPE[f.type]){
				  if (!(f instanceof Farmer_Satisficing)){
					  System.err.println ("UGBModelDisplay: Using wrong farmer Type");
					  return;
				  }
				  // cast to Farmer_Satisficing and call action method
				  Farmer_Satisficing fs = (Farmer_Satisficing)f;
				  fs.action();
			  }
			  else{
				  f.action();
			  }
		  }
	}
	
	protected void postStep(){ 	
		if(getYear() >= lastYear){
			schedule.executeEndActions();
			this.stop();
		}
	}
	
	
	///////////////////////////
	/// Display /////////////
	//////////////////////////
	
	protected void buildDisplay(){
		///search for PumpingQuotas for constructor of chart
		PumpingQuotas pq= null;
		Rule r;
		Iterator<Rule> itR = rules.iterator();
		while(itR.hasNext()){
			r = itR.next();
			if(r instanceof PumpingQuotas){
				pq = (PumpingQuotas) r;
				break;
			}
		}
		
		
		Iterator<ObserveFarmerChart> it = farmerCharts.iterator();
		ObserveFarmerChart chart;
		while(it.hasNext()){
			chart = it.next();
			
			chart.setFarmersToObserve(farmers);		
			chart.setXRange(0,40);
			if(chart instanceof SingleFarmerWaterChart){
				((SingleFarmerWaterChart) chart).setPumpingQuotas(pq);
			}
			chart.display();
		}
		
		Iterator<EtcChart> it2 = etcCharts.iterator();
		EtcChart etc;
		while(it2.hasNext()){
			etc = it2.next();
			etc.setObjectsToObserve(options);
			etc.setXRange(0,40);
			etc.display();
		}
	}
	
	/**This method is called in begin() and prepares the next model
	 * run in terms of deleting data in static variables.	 */
	public void clearData(){
		
		//set uses of Options to 0
		Farming.uses = new int[Farming.uses.length][Farming.uses[0].length];	
		farmers = null;
		options = null;
		rules = null;
	}
	
	
///////////////////////////////////////
	//Parameters that can be adjusted at the RePast GUI. (Getter and setter methods have to be
	//implemented, as well as parameter names have to be included in getInitParams(); see end of this class.
	
	//CHANGE: In the satisficing version only the relevant parameters will be displayed
	public String[] getInitParam() {
		
		if (GlobalVariables.SATISFICING_DISPLAY){
			String[] params = {"satisficing_PartTime","discardSmallPattern", "satisficing_Family", "satisficing_Business", "_profitThresholdIncrease", "_thresholdDecrease", "_historyAdjustmentOfThresholds", 
					"partTime_LabourLimit","partTime_SkillsShape","partTime_CapitalShape","partTime_FamilyLabour",
					"family_LabourLimit",  "family_SkillsShape",  "family_CapitalShape", "family_FamilyLabour",
					"business_LabourLimit","business_SkillsShape","business_CapitalShape","business_FamilyLabour",
					"writeToFile","diffusion", 
					};
			return params;
		}
		
		else{
		String[] params = {"satisficing_PartTime", "satisficing_Family", "satisficing_Business", "profitThresholdIncrease", "thresholdDecrease", "historyAdjustmentOfThresholds", "useBusinessParamsForAll","partTime_GrossMarginFactor", "partTime_RiskFactor","partTime_LegalFactor", "partTime_LabourShape","partTime_LabourLimit","partTime_SkillsShape","partTime_CapitalShape","partTime_FamilyLabour", 
				"family_GrossMarginFactor", "family_RiskFactor","family_LegalFactor", "family_LabourShape","family_LabourLimit","family_SkillsShape","family_CapitalShape", "family_FamilyLabour",
				"business_GrossMarginFactor", "business_RiskFactor","business_LegalFactor", "business_LabourShape","business_LabourLimit","business_SkillsShape","business_CapitalShape","business_FamilyLabour", 
				"waterPrice", "labourCosts", "diffusion","irrigation","writeToFile", "initType"};
		return params;
		}
		
	}
	/////////////////////////////////
	//getter und setter for params	
	///////////////////////////////
	
	
	public boolean getWriteToFile(){
		return writeToFile;
	}
	public void setWriteToFile(boolean b){
		writeToFile = b;
	}
	
	/**There are three init types {0,1,2}*/
	public int getInitType(){
		return GlobalVariables.init;
	}
	public void setInitType(int i){
		
		if(i>=0 && i<=2){
			GlobalVariables.init = i;
		}
	}
	
	///////////////////////////////////////////////
	//Part-time farmer
	// There will be some changes according to adjust to satisficing
	public double getPartTime_GrossMarginFactor(){
		return CompareLandUsePattern_Utility.GM_SHAPES[Farmer.PART_TIME];
	}
	public void setPartTime_GrossMarginFactor(double v){
		CompareLandUsePattern_Utility.GM_SHAPES[Farmer.PART_TIME] = v;
	}
	
	public double getPartTime_RiskFactor(){
		return CompareLandUsePattern_Utility.RISK_SHAPES[Farmer.PART_TIME];
	}
	public void setPartTime_RiskFactor(double v){
		CompareLandUsePattern_Utility.RISK_SHAPES[Farmer.PART_TIME] = v;
	}
	
	public double getPartTime_LegalFactor(){
		return CompareLandUsePattern_Utility.LEGAL_SHAPES[Farmer.PART_TIME];
	}
	public void setPartTime_LegalFactor(double v){
		CompareLandUsePattern_Utility.LEGAL_SHAPES[Farmer.PART_TIME] = v;
	}
	
	public double getPartTime_LabourShape(){
		return CompareLandUsePattern_Utility.LABOUR_SHAPES[Farmer.PART_TIME];
	}
	public void setPartTime_LabourShape(double v){
		CompareLandUsePattern_Utility.LABOUR_SHAPES[Farmer.PART_TIME] = v;
	}
	
	//CHANGE: include satisficing
	public double getPartTime_LabourLimit(){
		if (GlobalVariables.SATISFICING_PER_TYPE[0])
			return CompareLandUsePattern_Satisficing.LABOUR_LIMITS[Farmer.PART_TIME];
		else
			return CompareLandUsePattern_Utility.LABOUR_LIMITS[Farmer.PART_TIME];
	}
	
	//CHANGE: Include satisficing
	public void setPartTime_LabourLimit(double v){
		if (GlobalVariables.SATISFICING_PER_TYPE[0])
			CompareLandUsePattern_Satisficing.LABOUR_LIMITS[Farmer.PART_TIME] =v;
		else
			CompareLandUsePattern_Utility.LABOUR_LIMITS[Farmer.PART_TIME] = v;
	}
	
	public double getPartTime_FamilyLabour(){
		return Farmer.FAMILY_LABOUR[Farmer.PART_TIME];
	}
	public void setPartTime_FamilyLabour(double v){
		Farmer.FAMILY_LABOUR[Farmer.PART_TIME] = v;
	}
	
	//CHANGE: Include satisficing
	public double getPartTime_SkillsShape(){
		if (GlobalVariables.SATISFICING_PER_TYPE[0])
			return CompareLandUsePattern_Satisficing.SKILLS_SHAPES[Farmer.PART_TIME];
		else
			return CompareLandUsePattern_Utility.SKILLS_SHAPES[Farmer.PART_TIME];
	}
	
	//CHANGE: Include satisficing
	public void setPartTime_SkillsShape(double v){
		if (GlobalVariables.SATISFICING_PER_TYPE[0])
			CompareLandUsePattern_Satisficing.SKILLS_SHAPES[Farmer.PART_TIME] = v;
		else
			CompareLandUsePattern_Utility.SKILLS_SHAPES[Farmer.PART_TIME] = v;
	}
	
	//CHANGE: Include satisficing
	public double getPartTime_CapitalShape(){
		if (GlobalVariables.SATISFICING_PER_TYPE[0])
			return CompareLandUsePattern_Satisficing.CAPITAL_SHAPES[Farmer.PART_TIME];
		else
			return CompareLandUsePattern_Utility.CAPITAL_SHAPES[Farmer.PART_TIME];
	}
	
	//CHANGE: Include satisficing
	public void setPartTime_CapitalShape(double v){		
		if (GlobalVariables.SATISFICING_PER_TYPE[0])
			CompareLandUsePattern_Satisficing.CAPITAL_SHAPES[Farmer.PART_TIME] = v;
		else
			CompareLandUsePattern_Utility.CAPITAL_SHAPES[Farmer.PART_TIME] = v;
	}
		
///////////////////////////////////////////////
	//Family farmer
	public double getFamily_GrossMarginFactor(){
		return CompareLandUsePattern_Utility.GM_SHAPES[Farmer.FAMILY];
	}
	public void setFamily_GrossMarginFactor(double v){
		CompareLandUsePattern_Utility.GM_SHAPES[Farmer.FAMILY] = v;
	}
	
	public double getFamily_RiskFactor(){
		return CompareLandUsePattern_Utility.RISK_SHAPES[Farmer.FAMILY];
	}
	public void setFamily_RiskFactor(double v){
		CompareLandUsePattern_Utility.RISK_SHAPES[Farmer.FAMILY] = v;
	}
	
	public double getFamily_LegalFactor(){
		return CompareLandUsePattern_Utility.LEGAL_SHAPES[Farmer.FAMILY];
	}
	public void setFamily_LegalFactor(double v){
		CompareLandUsePattern_Utility.LEGAL_SHAPES[Farmer.FAMILY] = v;
	}
	
	public double getFamily_LabourShape(){
		return CompareLandUsePattern_Utility.LABOUR_SHAPES[Farmer.FAMILY];
	}
	public void setFamily_LabourShape(double v){
		CompareLandUsePattern_Utility.LABOUR_SHAPES[Farmer.FAMILY] = v;
	}
	
	//CHANGE: Include satisficing
	public double getFamily_LabourLimit(){
		if (GlobalVariables.SATISFICING_PER_TYPE[1])
			return CompareLandUsePattern_Satisficing.LABOUR_LIMITS[Farmer.FAMILY];
		else
			return CompareLandUsePattern_Utility.LABOUR_LIMITS[Farmer.FAMILY];
	}
	
	//CHANGE: Include satisficing
	public void setFamily_LabourLimit(double v){
		if (GlobalVariables.SATISFICING_PER_TYPE[1])
			CompareLandUsePattern_Satisficing.LABOUR_LIMITS[Farmer.FAMILY] = v;
		else
			CompareLandUsePattern_Utility.LABOUR_LIMITS[Farmer.FAMILY] = v;
	}
	
	public double getFamily_FamilyLabour(){
		return Farmer.FAMILY_LABOUR[Farmer.FAMILY];
	}
	public void setFamily_FamilyLabour(double v){
		Farmer.FAMILY_LABOUR[Farmer.FAMILY] = v;
	}
	
	//CHANGE: Include satisficing
	public double getFamily_SkillsShape(){
		if (GlobalVariables.SATISFICING_PER_TYPE[1])
			return CompareLandUsePattern_Satisficing.SKILLS_SHAPES[Farmer.FAMILY];
		else
			return CompareLandUsePattern_Utility.SKILLS_SHAPES[Farmer.FAMILY];
	}
	
	//CHANGE: Include satisficing
	public void setFamily_SkillsShape(double v){
		if (GlobalVariables.SATISFICING_PER_TYPE[1])
			CompareLandUsePattern_Satisficing.SKILLS_SHAPES[Farmer.FAMILY] = v;
		else
			CompareLandUsePattern_Utility.SKILLS_SHAPES[Farmer.FAMILY] = v;
	}
	
	//CHANGE: Include satisficing
	public double getFamily_CapitalShape(){
		if (GlobalVariables.SATISFICING_PER_TYPE[1])
			return CompareLandUsePattern_Satisficing.CAPITAL_SHAPES[Farmer.FAMILY];
		else
			return CompareLandUsePattern_Utility.CAPITAL_SHAPES[Farmer.FAMILY];
	}
	
	//CHANGE: Include satisficing
	public void setFamily_CapitalShape(double v){	
		if (GlobalVariables.SATISFICING_PER_TYPE[1])
			CompareLandUsePattern_Satisficing.CAPITAL_SHAPES[Farmer.FAMILY] = v;
		else
			CompareLandUsePattern_Utility.CAPITAL_SHAPES[Farmer.FAMILY] = v;
	}
	
///////////////////////////////////////////////
	//Business farmer
	//Business farmers are used to set values for all farmers if useBusinessParamsForAll is true
	
	public boolean useBusinessParamsForAll = false;
	
	public void setuseBusinessParamsForAll(boolean b){
		useBusinessParamsForAll = b;
	}
	public boolean getuseBusinessParamsForAll(){
		return useBusinessParamsForAll;
	}
	public double getBusiness_GrossMarginFactor(){
		return CompareLandUsePattern_Utility.GM_SHAPES[Farmer.BUSINESS];
	}
	public void setBusiness_GrossMarginFactor(double v){
		CompareLandUsePattern_Utility.GM_SHAPES[Farmer.BUSINESS] = v;
		if(useBusinessParamsForAll){
			setFamily_GrossMarginFactor(v);
			setPartTime_GrossMarginFactor(v);
			this.updateParameterPanel();
		}
	}
	
	public double getBusiness_RiskFactor(){
		return CompareLandUsePattern_Utility.RISK_SHAPES[Farmer.BUSINESS];
	}
	public void setBusiness_RiskFactor(double v){
		CompareLandUsePattern_Utility.RISK_SHAPES[Farmer.BUSINESS] = v;
		if(useBusinessParamsForAll){
			setFamily_RiskFactor(v);
			setPartTime_RiskFactor(v);
			this.updateParameterPanel();
		}
	}
	
	public double getBusiness_LegalFactor(){
		return CompareLandUsePattern_Utility.LEGAL_SHAPES[Farmer.BUSINESS];
	}
	public void setBusiness_LegalFactor(double v){
		CompareLandUsePattern_Utility.LEGAL_SHAPES[Farmer.BUSINESS] = v;
		if(useBusinessParamsForAll){
			setFamily_LegalFactor(v);
			setPartTime_LegalFactor(v);
			this.updateParameterPanel();
		}
	}
	
	public double getBusiness_LabourShape(){
		return CompareLandUsePattern_Utility.LABOUR_SHAPES[Farmer.BUSINESS];
	}
	
	public void setBusiness_LabourShape(double v){
		CompareLandUsePattern_Utility.LABOUR_SHAPES[Farmer.BUSINESS] = v;
		if(useBusinessParamsForAll){
			setFamily_LabourShape(v);
			setPartTime_LabourShape(v);
			this.updateParameterPanel();
		}
	}
	
	//CHANGE: Include satisficing
	public double getBusiness_LabourLimit(){
		if (GlobalVariables.SATISFICING_PER_TYPE[2])
			return CompareLandUsePattern_Satisficing.LABOUR_LIMITS[Farmer.BUSINESS];
		else
			return CompareLandUsePattern_Utility.LABOUR_LIMITS[Farmer.BUSINESS];
	}
	
	//CHANGE: Include satisficing
	public void setBusiness_LabourLimit(double v){
		if (GlobalVariables.SATISFICING_PER_TYPE[2])
			 CompareLandUsePattern_Satisficing.LABOUR_LIMITS[Farmer.BUSINESS] = v;
		else
			CompareLandUsePattern_Utility.LABOUR_LIMITS[Farmer.BUSINESS] = v;
		
		if(useBusinessParamsForAll){
			setFamily_LabourLimit(v);
			setPartTime_LabourLimit(v);
			this.updateParameterPanel();
		}
	}
	
	public double getBusiness_FamilyLabour(){
		return Farmer.FAMILY_LABOUR[Farmer.BUSINESS];
	}
	public void setBusiness_FamilyLabour(double v){
		Farmer.FAMILY_LABOUR[Farmer.BUSINESS] = v;
		if(useBusinessParamsForAll){
			setFamily_FamilyLabour(v);
			setPartTime_FamilyLabour(v);
			this.updateParameterPanel();
		}
	}
	
	//CHANGE: Include satisficing
	public double getBusiness_SkillsShape(){
		if (GlobalVariables.SATISFICING_PER_TYPE[2])
			return CompareLandUsePattern_Satisficing.SKILLS_SHAPES[Farmer.BUSINESS];
		else
			return CompareLandUsePattern_Utility.SKILLS_SHAPES[Farmer.BUSINESS];
	}
	
	//CHANGE: Include satisficing
	public void setBusiness_SkillsShape(double v){
		if (GlobalVariables.SATISFICING_PER_TYPE[2])
			 CompareLandUsePattern_Satisficing.SKILLS_SHAPES[Farmer.BUSINESS] = v;
		else
			CompareLandUsePattern_Utility.SKILLS_SHAPES[Farmer.BUSINESS] = v;
		
		if(useBusinessParamsForAll){
			setFamily_SkillsShape(v);
			setPartTime_SkillsShape(v);
			updateParameterPanel();
		}
	}
	
	//CHANGE: Include satisficing
	public double getBusiness_CapitalShape(){
		if (GlobalVariables.SATISFICING_PER_TYPE[2])
			return CompareLandUsePattern_Satisficing.CAPITAL_SHAPES[Farmer.BUSINESS];
		else
			return CompareLandUsePattern_Utility.CAPITAL_SHAPES[Farmer.BUSINESS];
	}
	
	//CHANGE: Include satisficing
	public void setBusiness_CapitalShape(double v){		
		if (GlobalVariables.SATISFICING_PER_TYPE[2])
			 CompareLandUsePattern_Satisficing.CAPITAL_SHAPES[Farmer.BUSINESS] = v;
		else
			CompareLandUsePattern_Utility.CAPITAL_SHAPES[Farmer.BUSINESS] = v;
		
		if(useBusinessParamsForAll){
			setFamily_CapitalShape(v);
			setPartTime_CapitalShape(v);
			this.updateParameterPanel();
		}
	}
	
	//Global params	
	public double getWaterPrice(){
		return GlobalVariables.waterPrice;
	}
	public void setWaterPrice(double v){
		GlobalVariables.setWaterPrice(v);
	}
	
	public double getLabourCosts(){
		return GlobalVariables.labourCosts;
	}
	public void setLabourCosts(double v){
		GlobalVariables.setLabourCosts(v);
	}
	
	public boolean getDiffusion(){
		return GlobalVariables.DIFFUSION;
	}
	public void setDiffusion(boolean b){
		GlobalVariables.DIFFUSION = b;
	}
	public boolean getIrrigation(){
		return GlobalVariables.IRRIGATION;
	}
	public void setIrrigation(boolean b){
		if((!GlobalVariables.IRRIGATION)&& b){
			GlobalVariables.IRRIGATION = b;
			if(options != null){
				startIrrigation();
			}
		}
		else{
			GlobalVariables.IRRIGATION = b;
		}
		
	}
	
	public boolean getSatisficing_PartTime (){
		return GlobalVariables.SATISFICING_PER_TYPE[0];
	}
	
	public void setSatisficing_PartTime(boolean b){
		GlobalVariables.SATISFICING_PER_TYPE[0] = b;
	}
	public boolean getSatisficing_Family (){
		return GlobalVariables.SATISFICING_PER_TYPE[1];
	}
	
	public void setSatisficing_Family(boolean b){
		GlobalVariables.SATISFICING_PER_TYPE[1] = b;
	}
	public boolean getSatisficing_Business (){
		return GlobalVariables.SATISFICING_PER_TYPE[2];
	}
	
	public void setSatisficing_Business(boolean b){
		GlobalVariables.SATISFICING_PER_TYPE[2] = b;
	}
	public double get_HistoryAdjustmentOfThresholds(){
		return Farmer_Satisficing.HISTORY_ADJUSTMENT_OF_THRESHOLDS;
	}
	
	public void set_HistoryAdjustmentOfThresholds(double d){
		Farmer_Satisficing.HISTORY_ADJUSTMENT_OF_THRESHOLDS = d;
	}
	
	public double get_ProfitThresholdIncrease(){
		return Farmer_Satisficing.PROFIT_THRESHOLD_INCREASE;
	}
	
	public void set_ProfitThresholdIncrease(double d){
		Farmer_Satisficing.PROFIT_THRESHOLD_INCREASE = d;
	}
	
	public double get_ThresholdDecrease(){
		return Farmer_Satisficing.THRESHOLD_DECREASE;
	}
	
	public void set_ThresholdDecrease(double d){
		Farmer_Satisficing.THRESHOLD_DECREASE = d;
	}
	
	public boolean getDiscardSmallPattern(){
		return GlobalVariables.DISCARDSMALLPATTERN;
	}
	
	public void setDiscardSmallPattern(boolean b){
		GlobalVariables.DISCARDSMALLPATTERN = b;
	}
	
	
	public void startIrrigation(){
//*debug*/		System.err.println("Start irrigation...");
		ArrayList<Option> newOptions = createOptions();
		//Add options in newOptions if not already in options
		Iterator<Option> itNew = newOptions.iterator();
		Iterator<Option> itOld;
		Option oN;
		boolean found;
		while(itNew.hasNext()){	//go through newOptions
			oN = itNew.next();
			found = false;
			itOld = options.iterator();
			while(itOld.hasNext()){	//if already there before
				if(itOld.next().equals(oN)){
					found = true;
					break;
				}
			}
			if(! found){ //new Option must be added
				options.add(oN);
			}
		}
		//set new Global Variables
		GlobalVariables.setUp(farmers, options);
		
/*debug*/	System.err.println(options);
		this.year = 1960;
		setXLabel("Irrigation introduced");
		
	}

	public int getYear(){
		return year;
	}
	
	public void setXLabel(String label){
		
		int x = (int) this.schedule.getCurrentTime();
		
		Iterator<ObserveFarmerChart> itFC = farmerCharts.iterator();
		ObserveFarmerChart c;
		while(itFC.hasNext()){
			c = itFC.next();
			if(c instanceof OpenGraph){
				((OpenGraph)c).setXTick(x, label);
			}
		}
		Iterator<EtcChart> itETC = etcCharts.iterator();
		EtcChart c2;
		while(itETC.hasNext()){
			c2 = itETC.next();
			if(c2 instanceof OpenGraph){
				((OpenGraph)c2).setXTick(x, label);
			}
		}
	}
	
	//////////////////////
	
	public String getName() {
		return "UGB model - display mode";
	}


	public Schedule getSchedule() {
		return schedule;
	}
	

	public void shuffleAgents() {
			SimUtilities.shuffle(farmers);
	}
	
	protected void updateParameterPanel(){
		
		try{
			
			uchicago.src.sim.util.ProbeUtilities.updateModelProbePanel();
		}catch(Exception e){
			//System.err.println(e); //do nothing 
		}


	}
}
