package model.decisionMaking;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.ListIterator;

import uchicago.src.sim.util.Random;

import model.*;

/**
 * This class implements a simple representation of the diffusion of innovations: it distinguishes
 * external influence (independent from #uses) from internal influence (depends on #uses).
 * 
 * @author Georg
 *
 */
public class CollectOptions_DiffusionNrPeople implements CollectOptions{

	public static double p1_default = 0.02;
	public static double p2_default = 0.1;
	
	/////////////////////////////////////////
	
	public ArrayList<Option> allOptions;
	
	/**p1: External influence (media, advertisement etc.), 
	 * p2: Internal influence (word of mouth)*/
	public double p1, p2;
	public int fullUsage;	//maximum spread of an Option
	
	/** Uses default values*/
	public CollectOptions_DiffusionNrPeople(){
		this(p1_default, p2_default, GlobalVariables.nrFarmersModel());
	}
	
	public CollectOptions_DiffusionNrPeople(double p1, double p2, int fullUsage){
		this.p1 = p1;
		this.p2 = p2;
		this.fullUsage = fullUsage;
	}
	
	public Option[] collectOptionsForFarmer(Farmer f){
		
		if(allOptions == null){
/*debug
			System.err.println("CollectOptions_DiffusionNrPeople.collectOptionsForFarmer: allOptions is null");
			return null;
/**/
		}
		
		HashSet<Option> cO = new HashSet<Option>();	//collects considered option
		
		Farming fa;
		//diffusion
		double p;
		ListIterator<Option> it = allOptions.listIterator();	
		while(it.hasNext()){
			fa= (Farming) it.next();
			if(!cO.contains(fa)){
				//if f knows the crop and the tech he considers to combine them
				if(f.knowsCrop(fa.getCrop().getType()) && (f.knowsTech(fa.getIrrigationTechnology().getType()))){
					cO.add(fa);
				}
				else{
					p = p1 + p2*(((double) fa.getUses())/fullUsage);
					if(Random.uniform.nextDouble()<p){
						cO.add(fa);
					}
				}
			}
		}
		return (Option[]) cO.toArray(new Option[cO.size()]);
	}
	
	/**l is used as list containing all options known to any farmer on which
	 * collectOptionsForFamers(f) operates.
	 * l is referenced, not copied. Therefore changes external to this
	 * class have an effect.*/
	public void setAllOptions(ArrayList<Option> l){
		allOptions = l;
	}
	
	public double getP1(){
		return p1;
	}
	public void setP1(double p1){
		this.p1 = p1;
	}
	public double getP2(){
		return p1;
	}
	public void setP2(double p2){
		this.p2 = p2;
	}
	public int getFullUsage(){
		return fullUsage;
	}
	public void setFullUsage(int fullUsage){
		this.fullUsage = fullUsage;
	}
	
}
