package model.decisionMaking;

import model.rules.*;

/**This class encapsulates the evaluation of a LandUsePattern by a rule.
 * It has been slightly modified by Marvin Nebel for the satisficing model.
 * @author Georg
 *
 */
public class EvaluatedLandUsePattern{

	public model.LandUsePattern pattern;
	
	public double capCompPayments;
	public boolean capSimplifiedScheme;
	/**0=no subscription, 1= 100% saving, 2 = 70% saving, 3= 50% saving, cf.  AEP.java*/
	public int aepOption;
	public double aepArea;
	public double aepCompPayments;
	public double waterLawPenalties;
	public boolean complyWithWaterLaw;
	// CHANGE: waterOverUse = max (0, 1-(Water Rights / Total Water Used)). 
	// Will be set in Pumping Quotas
	public double waterOverUse;
	public double irrigatedVine;
	public boolean complyWithVineLaw;
	
	/**Assumes compliance with laws as default*/
	public EvaluatedLandUsePattern(){
		//set some default values
		aepOption = AEP.NO_AEP;
		complyWithWaterLaw = true;
		complyWithVineLaw = true;
		waterOverUse = 0;
		irrigatedVine = 0;
	}
	
	public EvaluatedLandUsePattern(model.LandUsePattern p){

		this();
		pattern = p;
	}
	
	public double subsidies(){
		return capCompPayments + aepCompPayments;
	}
	public double penalties(){
		return waterLawPenalties;
	}
	//returns true if at least one law is violated
	public boolean violatesLaw(){
		return !(complyWithWaterLaw && complyWithVineLaw);
	}
	
	//CHANGE: returns a double that represents illegality. (1= legal behaviour)
	// a smaller number indicates a "more illegal" behaviour
	public double legality(){
		//assume legal behaviour
		double legality = 1.0;
		// if violating vine law, subtract 0.5 *Area of irrigated vine / total size
		if (!complyWithVineLaw){
			legality-= (0.5 * (irrigatedVine / pattern.size()));
		}
		// if violating waterLaw subtract 0.5 multiplied with the waterOverUse
		// waterOverUse = min(0, 1- (waterRights / Total Water Used))
		if (!complyWithWaterLaw){
			legality -= (waterOverUse * 0.5);
		}
		return legality;
	}
	
}
