package model.decisionMaking;

import java.util.ArrayList;
import java.util.List;
import java.util.Iterator;

import model.LandUsePattern;
import model.Farmer;

/**This class refines a landUsePattern in the way that minor "residual" 
 * landUses (landUse.size < 10% of f.farmSize) are deleted. A set of all possible refinements is provided (the
 * area of the deleted landUse is added to all other landUses, each time a new
 * pattern is created). The original landUsePattern is also maintained. 
 * 
 * @author Georg
 *
 */
public class RefinePattern {

	public ArrayList<EvaluatedLandUsePattern> refine(EvaluatedLandUsePattern elp, Farmer f){
		
		List<LandUsePattern> patterns = new ArrayList<LandUsePattern>(5);
		
		refine(patterns, elp.pattern,f);
		
		ArrayList<EvaluatedLandUsePattern> elpList = new ArrayList<EvaluatedLandUsePattern>(patterns.size());
		
		Iterator<LandUsePattern> it = patterns.iterator();
		while(it.hasNext()){
			elpList.add(new EvaluatedLandUsePattern(it.next()));
		}
		
		return elpList;
	}
	
	protected void refine(List<LandUsePattern> patterns,LandUsePattern lp, Farmer f){
		
		addIfNew(patterns,lp);	//add original Pattern if not already contained (due to multiple paths of the recursive function leading to same patterns)
		
		if(lp.length() >=2){ //if only one landUse --> nothing can be rescaled
	
			//re-scale all landUses that are smaller than 10% of total area and smaller than 1ha
			//shift area to all other landUses --> one new pattern each
			LandUsePattern refined;
			for(int i=0; i<lp.length();i++){
				double s = lp.get(i).size;
				if((s < f.sizeOfFarm*0.1) && (s < 1.0)){ //marginal areas are defined as <10% of farmsize and at the same time <1 ha
					for(int j=0; j<lp.length();j++){
						if(j!=i){
							refined = LandUsePattern.clone(lp);
							refined.get(j).size += refined.get(i).size;
							refined.remove(i);
							refine(patterns, refined,f);	//refined will be added in next recursion of refine()
						}
					}
				}
			}
		}
	}
	
	protected void addIfNew(List<LandUsePattern> patterns,LandUsePattern lp){
		
		LandUsePattern inList;
		for(int i=0; i<patterns.size(); i++){
			inList = patterns.get(i);
			if(inList.equals(lp)){	//lp already contained in patterns
				return;
			}
		}
		//no matching LandUsePattern found
		patterns.add(lp);
	}
}
