package model.decisionMaking;

import model.*;

public class RiskEvaluation {
	
	public static double riskFor(Farmer f, EvaluatedLandUsePattern elp){
		
		LandUsePattern lp = elp.pattern;
		LandUse l;
		Farming o;
		
		/*Aggregate areas dedicated to crops*/
		double cropSpecificVariation[] = new double[Crop.NR_CROPS];
		for(int i=0; i<lp.length(); i++){
			l = lp.get(i);
			if(l.option instanceof Farming){	//non-farming options currently not considered at all
				o = (Farming) l.option;
				cropSpecificVariation[o.getCrop().getType()]+=o.getCrop().getRelStddev(o.isIrrigated())*o.incomePerHa(l.size)*l.size;
			}
		}
		
		//calculate sqrt(sum(stddev^2)) over cropSpecificVariation
		double variationLp=0;
		
		for(int i=0; i<cropSpecificVariation.length; i++){
			if(! (i == Crop.FALLOW)){	//exclude Fallow, does not reduce risk
					variationLp += Math.pow(cropSpecificVariation[i],2.0);
			}
		}
		//no variability of yields and prices, thus no risk (e.g. only fallow)
		if(variationLp == 0.0){
			return 0.0;	
		}
			
		//else calculate risk
		variationLp = Math.sqrt(variationLp);
		
		double expectedGM = lp.income()+elp.subsidies() -elp.penalties() - lp.costs() - f.labourCosts(lp) ; 
		//if variationLP = expectedGM (and thus risk = Min(0.5 * variationLp / expectedGM, 1.0) = 0.5), 
		//assuming a normal distribution, this means in average approx. every third year results in losses.
		//Risk is defined to be max. 1.0. 
		//if expectedGM is 0 or negative, risk is set to 1.0. (if assuming a symmetric distribution of GM
		//around expectedGM=0, then in averaged every second year results in actual losses. This can be assumed to be
		//unbearable for farmers).
		if(expectedGM <= 0.0){
			return 1.0;
		}
		
		return  Math.min(variationLp / expectedGM, 1.0) ;		
	}

}
