package test;

import model.*;
import model.decisionMaking.*;
import model.rules.*;

import java.util.ArrayList;
import java.util.Iterator;

import uchicago.src.sim.util.Random;

public class TestAEP implements YearCount{

	protected ArrayList<Farmer> farmers;
	protected PumpingQuotas pq;
	protected AEP aep;
	Farmer f;
	LandUsePattern lp1985, lpAEP;
	
	protected int year = 1960;
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		
		Random.createNormal(1.0,1.0);
		Random.createUniform();
		
		TestAEP x = new TestAEP();
		x.setup();
		x.run();
		
	}
	
	public void run(){
		this.year = 1985;
		EvaluatedLandUsePattern elp = f.evaluatedLandUsePattern();
		System.out.println("1985:"+elp.pattern);
		//set pumping quotas
		pq.evaluateLandUsePattern(elp, f);
		System.out.println("entitled area: "+ pq.entitledArea(f, elp)+", water quota: "+pq.waterQuota(f, elp));
		f.setInitialLandUsePattern(lpAEP);
		System.out.println("1992:"+f.evaluatedLandUsePattern().pattern);	
		System.out.println("water used:"+lpAEP.waterUsed());
		EvaluatedLandUsePattern result;
		for(year = 1992; year <2005; year++){
			aep.evaluateLandUsePattern(f.evaluatedLandUsePattern(), f);
			result = f.evaluatedLandUsePattern();
			System.out.println(year+": chosen option "+result.aepOption+", payments/ha: "+result.aepCompPayments/result.aepArea+", aep area: "+result.aepArea);
		}	
	}
	
	public void setup(){
		//create farmer
		f = new Farmer(Farmer.BUSINESS, Farmer.BIG);
		farmers = new ArrayList<Farmer>();
		farmers.add(f);
		

		//create Options
		Crop tradCereals = new Crop(Crop.TRADITIONALCEREALS);
		Crop maize = new Crop(Crop.HIGHYIELDCEREALS);
		Crop fallow = new Crop(Crop.FALLOW);
		Crop vine = new Crop(Crop.VINEYARDS);
		Crop barley = new Crop(Crop.TRADITIONALCEREALS);
		Crop olives = new Crop(Crop.OLIVES);
		Crop sunflower = new Crop(Crop.SUNFLOWER);
		Crop sugarbeet = new Crop(Crop.SUGARBEET);
		Crop melons = new Crop(Crop.MELONS);
		Crop paprica = new Crop(Crop.PAPRICA);	
		
		IrrigationTechnology drip = new IrrigationTechnology("drip",IrrigationTechnology.DRIP, 0.9);
		IrrigationTechnology sprinkler =  new IrrigationTechnology("sprinkler",IrrigationTechnology.SPRINKLER, 0.75);
		IrrigationTechnology rainfed = new IrrigationTechnology("rainfed",IrrigationTechnology.RAINFED ,1.0);	
		IrrigationTechnology flood = new IrrigationTechnology("flood",IrrigationTechnology.FLOOD ,0.5);
		
		Farming sprinklerTradCereals = new Farming(tradCereals,sprinkler);
		Farming rainfedTradCereals = new Farming(tradCereals, rainfed);
		Farming sprinklerMaize = new Farming(maize, sprinkler);
		Farming rainfedFallow = new Farming(fallow, rainfed);
		Farming dripVine = new Farming(vine,drip);
		Farming sprinklerBarley = new Farming(barley, sprinkler);
		Farming rainfedOlives = new Farming(olives,rainfed);
		Farming rainfedSunflower = new Farming(sunflower, rainfed);
		Farming rainfedBarley = new Farming(barley, rainfed);
		Farming sprinklerSugarbeet = new Farming(sugarbeet, sprinkler);
		Farming floodMelons = new Farming(melons, flood);
		Farming dripPaprica = new Farming(paprica, drip);
		
		
		//create LandUsePattern
		LandUse l1  = new LandUse();
		l1.size = f.sizeOfFarm/3;
		l1.option = dripVine; 
		
		LandUse l2  = new LandUse();
		l2.size = f.sizeOfFarm-l1.size;
		l2.option = sprinklerBarley; 
		
		
		lp1985 = new LandUsePattern();
		lp1985.add(l1);
		lp1985.add(l2);
		f.setInitialLandUsePattern(lp1985);
		
		LandUse l3  = new LandUse();
		l3.size = l2.size*4278*0.5/2900*0.75;//f.sizeOfFarm;
		l3.option = sprinklerBarley; 
		
		LandUse l4  = new LandUse();
		l4.size = f.sizeOfFarm-l3.size;
		l4.option = rainfedFallow; 
		
		
		lpAEP = new LandUsePattern();
		lpAEP.add(l3);
		lpAEP.add(l4);
		
		
		System.out.println("Water price: "+GlobalVariables.waterPrice);
		System.out.println("Labour costs: "+GlobalVariables.labourCosts);
		
		pq = new PumpingQuotas();
		pq.setYearInfo(this);
		aep = new AEP(pq);
		aep.setYearInfo(this);
		
	}
	public int  getYear(){
		return year;
	}
	public void setXLabel(String label){
		//System.out.println(label);
	}
}
