package visualization;

import java.util.ArrayList;
import java.util.Iterator;

import model.Farmer;
import model.GlobalVariables;
import model.rules.AEP;

import uchicago.src.sim.analysis.OpenSequenceGraph;
import uchicago.src.sim.analysis.Sequence;
import uchicago.src.sim.engine.SimModel;

public class AepAreaChart extends OpenSequenceGraph implements ObserveFarmerChart {

	protected ArrayList<Farmer> farmers;
	protected boolean inited;
	
	public AepAreaChart(String title, SimModel model){
		super(title, model, GlobalVariables.getOutputDir()+"AEP_Area_"+title+GlobalVariables.dataFileExtension, uchicago.src.sim.analysis.PlotModel.CSV);
	}

	/**Sets farmers to be observed and creates AreaSequences to collect time series values.*/
	public void setFarmersToObserve(ArrayList<Farmer> farmers) {
		this.farmers = farmers;
		this.addSequence("Total area", new AreaSequence(-1));
		this.addSequence("Area 50%", new AreaSequence(AEP.OPTION50));
		this.addSequence("Area 70%", new AreaSequence(AEP.OPTION70));
		this.addSequence("Area 100%", new AreaSequence(AEP.OPTION100));
	}


/*	public void step() {

		this.display();
		super.step();
	}
*/	
	protected class AreaSequence implements Sequence {
			
		protected int option;
		
		/**This AreaSequence collects area under Option option. If Option is <0, total AEP area is
		 * collected.
		 * @param option
		 */
		public AreaSequence(int option){
			this.option = option;
		}
		
		public double getSValue(){
			double area = 0.0;
			Farmer f;
			Iterator<Farmer> it = farmers.iterator();
			while(it.hasNext()){
				f = it.next();
				if((this.option < 0)|| (this.option == f.evaluatedLandUsePattern().aepOption)){
					area += f.evaluatedLandUsePattern().aepArea*GlobalVariables.scalingFactor(f.sizeClass);
				}
			}
			return area;

		}
	}
}
