package visualization;

import java.util.ArrayList;
import java.util.Iterator;

import model.Farmer;
import model.GlobalVariables;
import uchicago.src.sim.analysis.OpenSequenceGraph;
import uchicago.src.sim.analysis.Sequence;
import uchicago.src.sim.engine.SimModel;

public class AepPaymentsChart extends OpenSequenceGraph implements ObserveFarmerChart {

	protected ArrayList<Farmer> farmers;
	protected Sequence payments;
	
	public AepPaymentsChart(String title, SimModel model){
		super(title, model, GlobalVariables.getOutputDir()+"AEP_Payments_"+title+GlobalVariables.dataFileExtension,uchicago.src.sim.analysis.PlotModel.CSV);
	}

	public void setFarmersToObserve(ArrayList<Farmer> farmers) {
		this.farmers = farmers;
	}


	public void step() {

		if(payments == null){
			payments = new PaymentsSequence();
			this.addSequence("Payments (1000 Euro)", payments);
			this.display();
		}
		this.display();
		super.step();
	}
	
	protected class PaymentsSequence implements Sequence {
				
		public double getSValue(){
			double payments = 0.0;
			Iterator<Farmer> it = farmers.iterator();
			Farmer f;
			while(it.hasNext()){
				f = it.next();
				payments += f.evaluatedLandUsePattern().aepCompPayments*GlobalVariables.scalingFactor(f.sizeClass);
			}
			return payments/1000.0; //Show with unit 1000Euro

		}
	}
}
