package visualization;

import java.io.*;
import java.util.StringTokenizer;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Comparator;
import java.util.Enumeration;

import model.Crop;
import model.IrrigationTechnology;

import java.util.Hashtable;
import java.util.ArrayList;

/** This class provides the possibility to visualize a model run whose data is stored in the
 * directory 'mainDir'. For each file in that dir a panel is launched which represents the respective
 * data.
 * @author gholtz
 *
 */


public class AggregateRuns{

	public static boolean workMultipleDirs = false;
	
	public static String inputMainDir = 
		"/home/student/m/mnebel/Simulationen/combinations/recombinedRuns/";
	
	
	
	
//	public static String dataSetName = "IrrigatedCropArea_Irrigated crops.mr.avg"; //should be in each subdir
	
	public static String[] toAggregate = 	{"Melons","Garlic","Paprica"}; 
											//{"Traditional cereals", "High yield cereals", "Sunflower"};
											//{"_flood", "_sprinkler", "_drip"};
	public static String outAppendix =  	"Horticulture"; 
											//"COP"; 
											//"_irrigated";
	
	////////////////////////////////////////////////////////////
	
	
	public String inputDir = null;
//	if no inputSubDirs are specified use all subDirs in mainDir
	//there may be no other subDirs in mainDir!!
//	if no outputFileDir is specified, use inputMainDir as default output dir
	public String outputDir = null; //"C:/Georg/Model/Analyse/Labour/";
	
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		
		
		if(workMultipleDirs){
			File mDir = new File(inputMainDir);
			File [] sDirs = mDir.listFiles(new FileFilter(){
				public boolean accept(File pathname){
					return pathname.isDirectory();
				}
			});
			
			AggregateRuns a;
			for(int i=0; i<sDirs.length; i++){
				System.err.println("working: "+sDirs[i].getName());
				a = new AggregateRuns();
				a.inputDir = inputMainDir.concat(sDirs[i].getName()).concat("/");
				try{
					a.run();
				}catch(Exception e){
					System.err.println(e);
				}
			}
		}
		else{
			try{
				new AggregateRuns().run();
			}catch(Exception e){
				System.err.println(e);
			}
		}
		
		
	}
	
	public int minYear = 1960, maxYear = 2010;
	
	
	public void run() throws Exception{
		
		Hashtable<String,ArrayList<BufferedReader>> inputReaders;
		Hashtable<String,String[]> labels = new Hashtable<String,String[]>();
		//label, writer
		Hashtable<String,BufferedWriter> outputWriters = null;
		
		Hashtable<BufferedReader,String[]> inLabelMemory = new Hashtable<BufferedReader, String[]>();
		
		//collect labels for output
		HashSet<String> outFiles = new HashSet<String>();
		
	
		//prepare inputFiles
		if(inputDir == null){
			inputDir = inputMainDir;
		}		
		inputReaders = createInputReaders();

		Enumeration<String> e = inputReaders.keys();
		while(e.hasMoreElements()){
			String outFileName = e.nextElement();

			//read labels of input files
			HashSet<String> dummy = new HashSet<String>();
			ArrayList<BufferedReader> al = inputReaders.get(outFileName);
			Iterator<BufferedReader> itR = al.iterator();
			while(itR.hasNext()){
				BufferedReader br = itR.next();
				String[] inLabels = readLabels(br);
				inLabelMemory.put(br,inLabels);
				for(int i=0;i<inLabels.length; i++){
					dummy.add(inLabels[i]);
				}
			}
			labels.put(outFileName,dummy.toArray(new String[dummy.size()]));
		}	
		//create output files
		try{
//			
			outputWriters = new Hashtable<String,BufferedWriter>();
			if(outputDir == null){
				outputDir = inputDir;
			}
			e = labels.keys();
System.err.println(labels.size());
			while(e.hasMoreElements()){
				String ofn = e.nextElement();
System.err.println("create "+ofn);

				File f = new File(outputDir.concat(ofn));
				if(! f.createNewFile()){
					System.err.println("can't create file "+outputDir.concat(ofn));
				}
				FileWriter fw = new FileWriter(f);
				BufferedWriter bw = new BufferedWriter(fw);
				//write subDirNames as headings
				bw.write("year, ");
				String [] labelNames = labels.get(ofn);
				for(int d=0; d<labelNames.length; d++){
					bw.write(labelNames[d]+", ");
				}
				bw.newLine();
				bw.flush();
				outputWriters.put(ofn,bw);
			}
		}catch(Exception ex){
			System.err.println(ex);
		}
		
System.err.println("output files created");
		for(int y=minYear; y<maxYear; y++){
			//read data lines
			//one output file at a time
			e = labels.keys();
			while(e.hasMoreElements()){
				String ofn = e.nextElement();	//name of outputfile
				String[] outLabels = labels.get(ofn);
				double[] sum = new double[outLabels.length];
				ArrayList<BufferedReader> brl = inputReaders.get(ofn); //get all readers of respective input
				Iterator<BufferedReader> itR = brl.iterator();
				while(itR.hasNext()){
					BufferedReader r = itR.next();
					double[] data = readDataLine(r);
					String[] inLabels  = inLabelMemory.get(r);
					for(int out =0; out<outLabels.length;out++){
						for(int in=0;in<inLabels.length;in++){
							if(outLabels[out].equals(inLabels[in])){
								sum[out]+=data[in];
							}
						}
					}
				}
				BufferedWriter bw = outputWriters.get(ofn);
				bw.write(y+", ");
				for(int i=0;i<sum.length;i++){
					bw.write(Double.toString(sum[i])+", ");
				}
				bw.newLine();
				bw.flush();
			}
			
		}

	}
	//for each subDir a reader for subDir/refDataSet is created
	protected Hashtable<String,ArrayList<BufferedReader>> createInputReaders() throws Exception{
		
		BufferedReader r;
		
		Hashtable<String,ArrayList<BufferedReader>> readers = new Hashtable<String,ArrayList<BufferedReader>>();
		
		//consider all files (not directories)
		File[] files = (new File(inputDir)).listFiles(new FileFilter(){
				public boolean accept(File pathname){
					return pathname.isFile();
				}
		});
			
		for(int i=0; i<files.length; i++){
			
			//check if filename contains one of "to Aggregate" Strings
			String fn = files[i].getName();
			for(int j=0; j<toAggregate.length; j++){	
				if (fn.contains(toAggregate[j])){		//found something to aggregate
					r = createReader(inputDir.concat(fn));										
					String outName = fn.replace(toAggregate[j],outAppendix);
					if(readers.get(outName)== null){
						readers.put(outName, new ArrayList<BufferedReader>());
					}
					readers.get(outName).add(r);
					
				}
			}
		}
		return readers;
	}
	
	protected BufferedReader createReader(String absoluteFileName){
		
		File f = new File(absoluteFileName);
		BufferedReader r = null;
		try{
			r = new BufferedReader(new FileReader(f));
		}catch(Exception e){
			System.err.println(e);
		}
		return r;
	}
	
	//reads labels for refRun
	protected String[] readLabels(BufferedReader refReader){
		
		String[] labels;
		
		StringTokenizer st;
		String line=null;
		
		//read intro (params etc.)
		do{
			try{
				line=refReader.readLine();
			}catch(Exception e){
				System.err.println(e);
			}
				if(line != null){
					if(line.startsWith("\"tick\"") || (line.startsWith("year"))){
						break;
					}
					if(line.length()==0){ //an empty line signals start of labels and data
						try{
							line=refReader.readLine();
						}catch(Exception e){
							System.err.println(e);
						}
						break;
					}		
				}
		}while(line!=null);	
		
		//create StringTokenizer for labels
		String delim = ",";
		st = new StringTokenizer(line,delim);
		String label = null;
//		st.nextToken(); //read away the label 'tick'	
		ArrayList<String> labelList = new ArrayList<String>();
		while(st.hasMoreTokens()){
			//remove '"' from labels
			label = st.nextToken().replace('"', ' ').trim();
			labelList.add(label);
		}
		//memorize labels in found order
		labels = labelList.toArray(new String[labelList.size()]);
		
		return labels;
	}
	
	protected double[] readDataLine(BufferedReader r){
		
		String delim = ",";
		StringTokenizer st = nextLine(r, delim);
		
		if(st == null){
			return null;
		}
		
		ArrayList<Double> dataList = new ArrayList<Double>();
		
		double d = nextDataToken(st);
		
		while(d>=0.0){
			dataList.add(new Double(d));
			d = nextDataToken(st);
		}
		Double dummy[] = dataList.toArray(new Double[dataList.size()]);	
		double data[] = new double[dummy.length];
		for(int i=0;i<dummy.length; i++){
			data[i] = dummy[i].doubleValue();
		}	
		return data;
		
	}
		
	public StringTokenizer nextLine(BufferedReader reader, String delim){
		
		String line =null;
		try{
			line=reader.readLine();
		}catch(Exception e){
			System.err.println(e);
			return null;
		}
		if(line == null){
			return null;
		}
		
		return new StringTokenizer(line,delim);
	}
	
	//returns -1.0 to signal end of line
	public double nextDataToken(StringTokenizer st){
		
		String token = null;
		if(st.hasMoreTokens()){
			try{
				token = st.nextToken();
			}catch(Exception e){
				System.err.println(e);
			}
			if(token.trim().length()==0){
				return 0.0;
			}
			else{
				return Double.valueOf(token);
			}
		}
		return -1.0;
	}
	
	/**value v in % of ref*/
	public double manipulateDataEntry(double ref, double v) throws Exception{
		if(ref==0.0){
			throw new Exception();
		}
		return v/ref*100;
	}

}
