package visualization;


import java.util.ArrayList;
import java.util.Iterator;

import model.Farmer;
import model.Farmer_Satisficing;
import model.GlobalVariables;
import uchicago.src.sim.analysis.OpenSequenceGraph;
import uchicago.src.sim.analysis.Sequence;
import uchicago.src.sim.analysis.plot.OpenGraph;
import uchicago.src.sim.engine.SimModel;
import visualization.SingleFarmerThresholdOutputChart.ScoreSequence;

/**This chart displays average Threshold and average Outputs for all farmers.
 * 
 * @author Marvin Nebel (mnebel@uos.de)
 * @version 03/29/2011
 *
 */
public class AverageThresholdOutputChart extends OpenSequenceGraph implements ObserveFarmerChart {

	protected ArrayList<Farmer> farmers;
	protected ArrayList<Farmer_Satisficing> toObserve;
	int nrOfFarmers = 0;
	public double refGM = 20000; //factor to downscale grossMargin in output

	
	
	public AverageThresholdOutputChart(String title, SimModel model) {
		super(title, model, GlobalVariables.getOutputDir()+"AverageThresholdOutput_"+title+GlobalVariables.dataFileExtension,uchicago.src.sim.analysis.PlotModel.CSV);
		toObserve = new ArrayList<Farmer_Satisficing>();
	}

	public void setFarmersToObserve(ArrayList<Farmer> a){
		farmers = a;
		Farmer f;
		Iterator <Farmer> it = farmers.iterator();
		while (it.hasNext()){
			f = it.next();
			if (f instanceof Farmer_Satisficing){
				toObserve.add((Farmer_Satisficing)f);
				nrOfFarmers++;
			}
		}
		
		if (Farmer_Satisficing.profitIncluded){
			this.addSequence("Output profit", new ThresholdOutputSequence(Farmer_Satisficing.PROFIT, true), OptionRepresentation.colorForOutput_Threshold(Farmer_Satisficing.PROFIT, true), OpenGraph.FILLED_SQUARE);
			this.addSequence("Threshold profit", new ThresholdOutputSequence(Farmer_Satisficing.PROFIT, false), OptionRepresentation.colorForOutput_Threshold(Farmer_Satisficing.PROFIT, false), OpenGraph.CROSS);
		}
		
		if (Farmer_Satisficing.labourIncluded){
			this.addSequence("Output labour", new ThresholdOutputSequence(Farmer_Satisficing.LABOUR, true), OptionRepresentation.colorForOutput_Threshold(Farmer_Satisficing.LABOUR, true), OpenGraph.FILLED_SQUARE);
			this.addSequence("Threshold labour", new ThresholdOutputSequence(Farmer_Satisficing.LABOUR, false), OptionRepresentation.colorForOutput_Threshold(Farmer_Satisficing.LABOUR, false), OpenGraph.CROSS);
		}
		
		if (Farmer_Satisficing.riskIncluded){
			this.addSequence("Output risk", new ThresholdOutputSequence(Farmer_Satisficing.RISK, true), OptionRepresentation.colorForOutput_Threshold(Farmer_Satisficing.RISK, true), OpenGraph.FILLED_SQUARE);
			this.addSequence("Threshold risk", new ThresholdOutputSequence(Farmer_Satisficing.RISK, false), OptionRepresentation.colorForOutput_Threshold(Farmer_Satisficing.RISK, false), OpenGraph.CROSS);
		}
		
		if (Farmer_Satisficing.legalityIncluded){
			this.addSequence("Output legality", new ThresholdOutputSequence(Farmer_Satisficing.LEGALITY, true), OptionRepresentation.colorForOutput_Threshold(Farmer_Satisficing.LEGALITY, true), OpenGraph.FILLED_SQUARE);
			this.addSequence("Threshold legality", new ThresholdOutputSequence(Farmer_Satisficing.LEGALITY, false), OptionRepresentation.colorForOutput_Threshold(Farmer_Satisficing.LEGALITY, false), OpenGraph.CROSS);
		}
		
		
	}
	
	/* Displays the average Threshold and Output of all farmer_satisficing */
	protected class ThresholdOutputSequence implements Sequence {
		

		int p; boolean output;
		
		public ThresholdOutputSequence(int t, boolean b){
			p = t;
			output = b;
		}
		
		public double getSValue() {
			return getValue(toObserve, p, output);
		}
		
		protected double getValue (ArrayList<Farmer_Satisficing>toObserve, int priority, boolean output){
			Farmer_Satisficing fs;
			double value = 0.0;
					
			if(toObserve != null){
				Iterator<Farmer_Satisficing> it = toObserve.iterator();
				// go through all farmer_satisficing sum up their output or their thresholds
				while(it.hasNext()){
					fs= it.next();
					if(output){
						value+= fs.getCurrentOutput()[priority];
					}
					else{
						value += fs.getThreshold()[priority];
					}
				}
			}
			//downscale profit
			if(priority == Farmer_Satisficing.PROFIT){
				return value/(refGM*nrOfFarmers);
			}
			else return value/nrOfFarmers;
			
		}
		
	}
}
