package visualization;

import java.util.ArrayList;
import java.util.Iterator;

import uchicago.src.sim.analysis.OpenSequenceGraph;
import uchicago.src.sim.analysis.Sequence;
import uchicago.src.sim.analysis.plot.OpenGraph;
import uchicago.src.sim.engine.SimModel;

import model.Farmer;
import model.LandUse;
import model.Crop;
import model.Farming;
import model.LandUsePattern;
import model.GlobalVariables;

public class CropCategoryAreaChart extends OpenSequenceGraph implements ObserveFarmerChart{

	protected ArrayList<Farmer> toObserve;
	protected double areas[];
	protected boolean inited = false;
	
//	protected int[] types;
	
	public CropCategoryAreaChart(String title, SimModel model){
		super(title, model,GlobalVariables.getOutputDir()+"CropCategoryArea_"+title+GlobalVariables.dataFileExtension,uchicago.src.sim.analysis.PlotModel.CSV);
	}
	
	public void setFarmersToObserve(ArrayList<Farmer> f){
		toObserve = f;
	}
	
	public void step(){
		Farmer f;
		
		if (!inited){
			this.addSequence("COP", new CropAreaSequence(Crop.COP),OptionRepresentation.colorForCropType(Crop.TRADITIONALCEREALS),OpenGraph.CROSS);
			this.addSequence("OFC", new CropAreaSequence(Crop.OFC),OptionRepresentation.colorForCropType(Crop.SUGARBEET),OpenGraph.CROSS);
			this.addSequence("Horticulture", new CropAreaSequence(Crop.HORTICULTURE),OptionRepresentation.colorForCropType(Crop.MELONS),OpenGraph.CROSS);
			this.addSequence("Olives", new CropAreaSequence(Crop.OLIVES),OptionRepresentation.colorForCropType(Crop.OLIVES),OpenGraph.CROSS);
			this.addSequence("Vineyards", new CropAreaSequence(Crop.VINEYARDS),OptionRepresentation.colorForCropType(Crop.VINEYARDS),OpenGraph.CROSS);
			inited = true;
		}
		//sum up areas of crops
		if(toObserve != null){
			areas = new double[10];
			Iterator<Farmer> it = toObserve.iterator();
			double scalingFactor;
			while(it.hasNext()){
				f = it.next();
				scalingFactor = GlobalVariables.scalingFactor(f.sizeClass); 
				LandUsePattern p = f.landUsePattern();
				LandUse l;
				int t;
				for(int i=0; i<p.length(); i++){
					l = p.get(i);
					t = ((Farming)l.option).getCrop().getType();
					if(t==Crop.TRADITIONALCEREALS || t==Crop.HIGHYIELDCEREALS || t==Crop.SUNFLOWER || t==Crop.FALLOW){
						areas[Crop.COP]+= l.size*scalingFactor;
					}
					else if(t==Crop.POTATOES || t == Crop.SUGARBEET){
						areas[Crop.OFC]+= l.size*scalingFactor;
					}
					else if(t==Crop.MELONS || t == Crop.PAPRICA || t==Crop.GARLIC){
						areas[Crop.HORTICULTURE]+= l.size*scalingFactor;
					}
					else if(t==Crop.VINEYARDS){
						areas[Crop.VINEYARDS]+= l.size*scalingFactor;
					}else if(t==Crop.OLIVES){
						areas[Crop.OLIVES]+= l.size*scalingFactor;
					}
				}
			}
			super.step();
			this.display();
		}
	}
	
	protected class CropAreaSequence implements Sequence {
		
		protected int type;
		
		public CropAreaSequence(int t){
			type = t;
		}
		//crop area in % of UAA
		public double getSValue() {
		
			return areas[type] / GlobalVariables.UAA*100;
		}
	}
}
