package visualization;

import java.util.Hashtable;
import java.util.ArrayList;
import java.util.Iterator;

import uchicago.src.sim.analysis.OpenSequenceGraph;
import uchicago.src.sim.analysis.Sequence;
import uchicago.src.sim.analysis.plot.OpenGraph;
import uchicago.src.sim.engine.SimModel;

import model.Farmer;
import model.Crop;
import model.LandUsePattern;
import model.GlobalVariables;
import model.Farming;
import model.LandUse;

public class FarmerSizeClassCropChart extends OpenSequenceGraph implements ObserveFarmerChart{

	protected ArrayList<Farmer> toObserve;
	protected Hashtable<String,CropSequence> sequences;
	protected double minSize, maxSize;
	
	/**minSize and maxSize allows to restrict the graph to a group of farmers having farmsizes
	 * in the interval [minSize, maxSize].
	 * If the interval does not make sense, the interval [0,Double.Max_Value] is chosen as default.
	 */
	public FarmerSizeClassCropChart(String title, SimModel model, double minSize, double maxSize){
		super(title, model, GlobalVariables.getOutputDir()+"FarmSizeClassCrop_"+title+GlobalVariables.dataFileExtension,uchicago.src.sim.analysis.PlotModel.CSV);
		sequences = new Hashtable<String,CropSequence>();
		if((minSize>=0) && (maxSize>=0) && (minSize < maxSize)){
			this.minSize = minSize; 
			this.maxSize = maxSize;
		}
		else{
			this.minSize =0;
			this.maxSize = Double.MAX_VALUE;
		}
	}
	
	public void setFarmersToObserve(ArrayList<Farmer> fa){
		toObserve = new ArrayList<Farmer>();
		Iterator<Farmer> it = fa.iterator();
		Farmer f;
		while(it.hasNext()){
			f = it.next();
			double size = f.sizeOfFarm;
			if( (size >= minSize) && (size<maxSize)){
				toObserve.add(f);
			}
		}
	}
	
	public void step(){
		
		if(toObserve != null){
			Farmer f;
			Iterator<Farmer> it = toObserve.iterator();
			while(it.hasNext()){
				f = it.next();
				LandUsePattern p = f.landUsePattern();
				Crop c;
				CropSequence s;
				Farming o;
				for(int i=0; i<p.length(); i++){
					o = ((Farming) (p.get(i).option));
					c = o.getCrop();
					if(! sequences.containsKey(c.getName())){ //use name of LandUse since objects are cloned from time to time
						s= new CropSequence(c);
						sequences.put(c.getName(), s);
						this.addSequence(c.getName(), s, OptionRepresentation.colorForOption(o),OpenGraph.CROSS);
						this.display();
					}
				}
			}
		}
		super.step();

	}
	
	protected class CropSequence implements Sequence {
		
		protected Crop crop;
		
		public CropSequence(Crop c){
			crop = c;
		}
		
		public double getSValue() {
			
			Farmer f;
			Iterator<Farmer> it = toObserve.iterator();
			Crop c;
			double size =0;
			while(it.hasNext()){
				f = it.next();
				LandUsePattern p = f.landUsePattern();
				LandUse l;
				
				for(int i=0; i<p.length(); i++){
					l = p.get(i);
					c = ((Farming)l.option).getCrop();
					if(c.equals(crop)){
						size+=l.size*GlobalVariables.scalingFactor(f.sizeClass);
					}
				}
			}
			return size;
		}
	}
}
