package visualization;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;

import model.Farmer;
import model.GlobalVariables;
import model.IrrigationTechnology;
import model.LandUsePattern;
import model.LandUse;
import model.Farming;
import model.Crop;

import uchicago.src.sim.analysis.OpenSequenceGraph;
import uchicago.src.sim.analysis.Sequence;
import uchicago.src.sim.engine.SimModel;

public class IrrigatedAreaPerTechChart extends OpenSequenceGraph implements ObserveFarmerChart {

	protected ArrayList<Farmer> farmers;
	//indices: 0=TotalArea, 1=Flood, 2=Sprinkler, 3=Drip
	protected double area[];
	protected boolean inited = false;
//	protected Hashtable<String,AreaSequence> area;
	
	public IrrigatedAreaPerTechChart(String title, SimModel model){
		super(title, model, GlobalVariables.getOutputDir()+"IrrigatedAreaPerTech_"+title+GlobalVariables.dataFileExtension, uchicago.src.sim.analysis.PlotModel.CSV);
	}

	public void setFarmersToObserve(ArrayList<Farmer> farmers) {
		this.farmers = farmers;
	}


	public void step() {

		if(! inited){
//		if(area == null){
			AreaSequence s;
//			area = new Hashtable<String,AreaSequence>();
//			area.put("Flood irrigation", value)
			this.addSequence("Flood irrigation (ha)", new AreaSequence(IrrigationTechnology.FLOOD));
			this.addSequence("Sprinkler (ha)", new AreaSequence(IrrigationTechnology.SPRINKLER));
			this.addSequence("Drip irrigation (ha)", new AreaSequence(IrrigationTechnology.DRIP));
			this.addSequence("Total irrigated area (ha)", new AreaSequence(0));
			this.display();
			inited = true;
		}
		area = new double[4];
		double a;
		Farmer f;
		LandUsePattern lp;
		LandUse l;
		IrrigationTechnology t;
		Iterator<Farmer> it = farmers.iterator();
		while(it.hasNext()){
			f = it.next();
			lp = f.landUsePattern();
			for(int i=0; i<lp.length(); i++){
				l = lp.get(i);
				t= ((Farming)l.option).getIrrigationTechnology();
				if( ((Farming)l.option).getCrop().getType()!= Crop.FALLOW){ //exclude Fallow
					if((t!= null) && (t.getType() != IrrigationTechnology.RAINFED)){
						a= l.size*GlobalVariables.scalingFactor(f.sizeClass);
						area[0] +=a;	//add to total irrigated area
						area[t.getType()] +=a;
					}
				}
			}
		}
		this.display();
		super.step();
	}
	
	protected class AreaSequence implements Sequence {
			
		protected int irrigationType;
		public AreaSequence(int irrigationType){
			this.irrigationType = irrigationType;
		}
		public double getSValue(){
			
			return area[irrigationType];

		}
	}
}
