package visualization;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;

import model.Farmer;
import model.GlobalVariables;
import model.IrrigationTechnology;
import model.LandUsePattern;
import model.LandUse;
import model.Farming;
import model.Crop;

import uchicago.src.sim.analysis.OpenSequenceGraph;
import uchicago.src.sim.analysis.Sequence;
import uchicago.src.sim.analysis.plot.OpenGraph;
import uchicago.src.sim.engine.SimModel;

public class IrrigatedCropAreaChart extends OpenSequenceGraph implements ObserveFarmerChart {

	protected ArrayList<Farmer> farmers;
	//indices: cf. Croptypes
	protected double area[];
	protected boolean inited = false;
	
	public IrrigatedCropAreaChart(String title, SimModel model){
		super(title, model,GlobalVariables.getOutputDir()+"IrrigatedCropArea_"+title+GlobalVariables.dataFileExtension,uchicago.src.sim.analysis.PlotModel.CSV);
	}

	public void setFarmersToObserve(ArrayList<Farmer> farmers) {
		this.farmers = farmers;
	}


	public void step() {

		if(! inited){
			this.addSequence("High yield cereals", new CropAreaSequence(Crop.HIGHYIELDCEREALS),OptionRepresentation.colorForCropType(Crop.HIGHYIELDCEREALS),OpenGraph.CROSS);
			this.addSequence("Traditional cereals", new CropAreaSequence(Crop.TRADITIONALCEREALS),OptionRepresentation.colorForCropType(Crop.TRADITIONALCEREALS),OpenGraph.CROSS);
			this.addSequence("Sunflower", new CropAreaSequence(Crop.SUNFLOWER),OptionRepresentation.colorForCropType(Crop.SUNFLOWER),OpenGraph.CROSS);
			this.addSequence("Vineyards", new CropAreaSequence(Crop.VINEYARDS),OptionRepresentation.colorForCropType(Crop.VINEYARDS),OpenGraph.CROSS);
			this.addSequence("Sugarbeet", new CropAreaSequence(Crop.SUGARBEET),OptionRepresentation.colorForCropType(Crop.SUGARBEET),OpenGraph.CROSS);
			this.addSequence("Melons", new CropAreaSequence(Crop.MELONS),OptionRepresentation.colorForCropType(Crop.MELONS),OpenGraph.CROSS);
			this.addSequence("Paprica", new CropAreaSequence(Crop.PAPRICA),OptionRepresentation.colorForCropType(Crop.PAPRICA),OpenGraph.CROSS);
			this.addSequence("Olives", new CropAreaSequence(Crop.OLIVES),OptionRepresentation.colorForCropType(Crop.OLIVES),OpenGraph.CROSS);
			this.addSequence("Garlic", new CropAreaSequence(Crop.GARLIC),OptionRepresentation.colorForCropType(Crop.GARLIC),OpenGraph.CROSS);
			this.display();
			inited = true;
		}
		area = new double[11];
		double a;
		Farmer f;
		LandUsePattern lp;
		LandUse l;
		IrrigationTechnology t;
		Farming o;
		Iterator<Farmer> it = farmers.iterator();
		while(it.hasNext()){
			f = it.next();
			lp = f.landUsePattern();
			for(int i=0; i<lp.length(); i++){
				l = lp.get(i);
				o = (Farming)l.option;
				t= o.getIrrigationTechnology();
					if((t!= null) && (t.getType() != IrrigationTechnology.RAINFED)){
						a= l.size*GlobalVariables.scalingFactor(f.sizeClass);
						area[o.getCrop().getType()] +=a;
					}
				}
		}
		this.display();
		super.step();
	}
	
	protected class CropAreaSequence implements Sequence {
			
		protected int cropType;
		public CropAreaSequence(int cropType){
			this.cropType = cropType;
		}
		public double getSValue(){
			
			return area[cropType];

		}
	}
}
