package visualization;

import java.io.*;
import java.util.StringTokenizer;

import model.Crop;
import model.IrrigationTechnology;

import java.util.Hashtable;
import java.util.ArrayList;

/** This class provides the possibility to visualize a model run whose data is stored in the
 * directory 'mainDir'. For each file in that dir a panel is launched which represents the respective
 * data.
 * @author gholtz
 *
 */


public class NormalizeRunsToRefRun{

	public static String inputMainDir = 
		"C:/Georg/Model/Analyse/mixedParams_HomogenousFarmers/runs/";
	
	public static String[] inputSubDirs = null;//{"_45/","_46/","_47/","_48/","_49/"}; //null; //{"labourLimit_1_5/ModelOutput/","labourLimit_1_75/ModelOutput/","labourLimit_2/ModelOutput/",
		//"labourLimit_2_25/ModelOutput/","labourLimit_2_5/ModelOutput/"};
	
	public static String refDataSet = "TotalLandUsePattern_Total landUse.mr.avg.avg"; //must be in inputMainDir
	public static String dataSet = "TotalLandUsePattern_Total landUse.mr.avg"; //must exist in each subDir
	
	public static String outputFileDir = null;//"C:/Georg/Model/Analyse/Labour/relatedRuns/";
	
	
	////////////////////////////////////////////////////////////
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		
		new NormalizeRunsToRefRun().run();
		
	}
	
	public int minYear = 1960, maxYear = 2010;
	
	
	public void run(){
	
		BufferedReader refReader;
		double[] refDataLine;
		String[] refLabels;
		
		
		Hashtable<String,BufferedReader> inputReaders;
		Hashtable<String,String[]> inputLabels;
		Hashtable<String, double[]> inputDataLine;
		
		Hashtable<String,BufferedWriter> outputWriters;
		
		//read ref run
		refReader = createReader(inputMainDir.concat(refDataSet));
		refLabels = readLabels(refReader);
		
		//prepare inputFiles
		inputReaders = createInputReaders();
		inputLabels = new Hashtable<String,String[]>();
		for(int i=0; i<inputSubDirs.length;i++){
			inputLabels.put(inputSubDirs[i],readLabels(inputReaders.get(inputSubDirs[i])));
		}
		
		//create outputFiles and writers
		//output data has same order as ref run
		outputWriters = new Hashtable<String,BufferedWriter>();
		try{
			if(outputFileDir == null){
				outputFileDir = inputMainDir.concat("relatedRuns/");
				if(! (new File(outputFileDir)).mkdir()){
					System.err.println("can't create output directory!");
				}
			}			
			for(int i=0; i<refLabels.length;i++){
				File f = new File(outputFileDir.concat(refLabels[i])+".ref");
				if(! f.createNewFile()){
					System.err.println("can't create file "+outputFileDir.concat(refLabels[i]));
				}
				FileWriter fw = new FileWriter(f);
				BufferedWriter bw = new BufferedWriter(fw);
				//write subDirNames as headings
				bw.write("year, ");
				for(int d=0; d<inputSubDirs.length; d++){
					bw.write(inputSubDirs[d]+", ");
				}
				bw.newLine();
				bw.flush();
				outputWriters.put(refLabels[i],bw);
			}
		}catch(Exception e){
			System.err.println(e);
		}
		
		for(int y=minYear; y<=maxYear; y++){
			//read data lines
			refDataLine = readDataLine(refReader);
			inputDataLine = new Hashtable<String, double[]>();
			for(int i=0; i<inputSubDirs.length;i++){
				inputDataLine.put(inputSubDirs[i],readDataLine(inputReaders.get(inputSubDirs[i])));
			}
			
			//go through all subDir data
			String d;
			String[] labels; 
			double[] data;
			
			try{
			
				for(int j=0;j<refLabels.length;j++){
					outputWriters.get(refLabels[j]).write(y+", ");
				}
				
				for(int i=0; i<inputSubDirs.length; i++){
					d= inputSubDirs[i];
					labels = inputLabels.get(d);
					data = inputDataLine.get(d);
					//go over labels
					for(int j=0;j<refLabels.length;j++){
						//lookup correct data entry
						BufferedWriter w = outputWriters.get(refLabels[j]);
						for(int k=0;k<labels.length;k++){
							if(refLabels[j].equals(labels[k])){ //found correct
								try{
									double rel = manipulateDataEntry(refDataLine[j],data[k]);
									w.write(Double.toString(rel));
								}catch(Exception e){
										w.write(", ");
									
								}
							}								
						}
						w.write(", ");
					}
				}
				BufferedWriter w;
				for(int j=0;j<refLabels.length;j++){
					w = outputWriters.get(refLabels[j]);
					w.newLine();
					w.flush();
				}
			}catch(Exception e){
				System.err.println(e);
			}
		}

	}
	//for each subDir a reader for subDir/refDataSet is created
	protected Hashtable<String,BufferedReader> createInputReaders(){
		

		BufferedReader r;
		Hashtable<String,BufferedReader> readers = new Hashtable<String,BufferedReader>();
		

		//if no inputSubDirs are specified use all subDirs in mainDir
		//there may be no other subDirs in mainDir!!
		if(inputSubDirs == null){
			//create array of all Files in inputMainDir which are directories
			File[] sDirFiles = (new File(inputMainDir)).listFiles(new FileFilter(){
				public boolean accept(File pathname){
					return pathname.isDirectory();
				}
			});
			inputSubDirs = new String[sDirFiles.length];
			for(int i=0; i<sDirFiles.length; i++){
				inputSubDirs[i] = sDirFiles[i].getName()+"/";
			}
		}
		
		
		for(int i=0; i<inputSubDirs.length; i++){
			r = createReader(inputMainDir.concat(inputSubDirs[i]).concat(dataSet));
			readers.put(inputSubDirs[i], r);
		}
		return readers;
	}
	
	protected BufferedReader createReader(String absoluteFileName){
		
		File f = new File(absoluteFileName);
		BufferedReader r = null;
		try{
			r = new BufferedReader(new FileReader(f));
		}catch(Exception e){
			System.err.println(e);
		}
		return r;
	}
	
	//reads labels for refRun
	protected String[] readLabels(BufferedReader refReader){
		//scan refRun and get labels and data
		
		String[] labels;
		
		StringTokenizer st;
		String line=null;
		
		//read intro (params etc.)
		do{
			try{
				line=refReader.readLine();
			}catch(Exception e){
				System.err.println(e);
			}
				if(line != null){
					//if(line.startsWith("\"tick\"")){
					if(line.length()==0){ //an empty line signals start of labels and data
						try{
							line=refReader.readLine();
						}catch(Exception e){
							System.err.println(e);
						}
//*debug*/System.err.println("label line:"+line);
						break;
					}
				}
		}while(line!=null);	
		
		//create StringTokenizer for labels
		String delim = ",";
		st = new StringTokenizer(line,delim);
		String label = null;
//		st.nextToken(); //read away the label 'tick'	
		ArrayList<String> labelList = new ArrayList<String>();
		while(st.hasMoreTokens()){
			//remove '"' from labels
			label = st.nextToken().replace('"', ' ').trim();
			labelList.add(label);
		}
		//memorize labels in found order
		labels = labelList.toArray(new String[labelList.size()]);
		
		return labels;
	}
	
	protected double[] readDataLine(BufferedReader r){
		
		String delim = ",";
		StringTokenizer st = nextLine(r, delim);
		ArrayList<Double> dataList = new ArrayList<Double>();
		
		double d = nextDataToken(st);
		
		while(d>=0.0){
			dataList.add(new Double(d));
			d = nextDataToken(st);
		}
		Double dummy[] = dataList.toArray(new Double[dataList.size()]);	
		double data[] = new double[dummy.length];
		for(int i=0;i<dummy.length; i++){
			data[i] = dummy[i].doubleValue();
		}	
		return data;
		
	}
		
	public StringTokenizer nextLine(BufferedReader reader, String delim){
		
		String line =null;
		try{
			line=reader.readLine();
		}catch(Exception e){
			System.err.println(e);
			return null;
		}
		
		return new StringTokenizer(line,delim);
	}
	
	//returns -1.0 to signal end of line
	public double nextDataToken(StringTokenizer st){
		
		String token = null;
		if(st.hasMoreTokens()){
			try{
				token = st.nextToken();
			}catch(Exception e){
				System.err.println(e);
			}
			if(token.trim().length()==0){
				return 0.0;
			}
			else{
				return Double.valueOf(token);
			}
		}
		return -1.0;
	}
	
	/**value v in % of ref*/
	public double manipulateDataEntry(double ref, double v) throws Exception{
		if(ref==0.0){
			throw new Exception();
		}
		return v/ref*100;
	}

}
