package visualization;

import java.io.*;
import java.util.StringTokenizer;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Comparator;
import java.util.Enumeration;

import model.Crop;
import model.IrrigationTechnology;

import java.util.Hashtable;
import java.util.ArrayList;

/** This class rearranges model output of several runs in the way that similar data sets (e.g. area of flood irrgated vineyards) 
 * are stored in one file.
 * @author gholtz
 *
 */


public class RecombineRuns{

	public static boolean workMultipleDirs = false;
	
	public static String inputMainDir = 
		"/home/student/m/mnebel/Simulationen/combinations/";
	
	
	
	
	public static String dataSetName = 	//"WaterExtractions_Water extracted from aquifer.mr.avg"; //should be in each subdir
										//"AverageThresholdOutput_Average threshold and output.mr.avg";
										"IrrigatedCropArea_Irrigated crops.mr.avg";
	
	
	////////////////////////////////////////////////////////////
	
	
	public String inputDir = null;
//	if no inputSubDirs are specified use all subDirs in mainDir
	//there may be no other subDirs in mainDir!!
	public String[] inputSubDirs = null;// {"_45/","_46/","_47/","_48/","_49/"};//null;//
//	if no outputFileDir is specified, use inputMainDir as default output dir
	public String outputFileDir = null; //"C:/Georg/Model/Analyse/Labour/";
	
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		
		
		if(workMultipleDirs){
			File mDir = new File(inputMainDir);
			File [] sDirs = mDir.listFiles(new FileFilter(){
				public boolean accept(File pathname){
					return pathname.isDirectory();
				}
			});
			
			RecombineRuns a;
			for(int i=0; i<sDirs.length; i++){
				System.err.println("working: "+sDirs[i].getName());
				a = new RecombineRuns();
				a.inputDir = inputMainDir.concat(sDirs[i].getName()).concat("/");
				a.run();
			}
		}
		else{
			new RecombineRuns().run();
		}
		
		
	}
	
	public int minYear = 1960, maxYear = 2010;
	
	
	public void run(){
		
		//inputDir, reader
		Hashtable<String,BufferedReader> inputReaders;
		
		//inputDir, labels
		Hashtable<String,String[]> inputLabels;
		
		//inputDir, data
		Hashtable<String, double[]> inputDataLine;
		
		//label, writer
		Hashtable<String,BufferedWriter> outputWriters = null;
		
		//collect labels for output
		HashSet<String> outLabels = new HashSet<String>();
		
	
		//prepare inputFiles
		
		if(inputDir == null){
			inputDir = inputMainDir;
		}
		
		inputReaders = createInputReaders();

		inputLabels = new Hashtable<String,String[]>();
		//read labels of input files
		for(int i=0; i<inputSubDirs.length;i++){	
			String[] inLabels = readLabels(inputReaders.get(inputSubDirs[i]));

			inputLabels.put(inputSubDirs[i],inLabels);
			//save labels for output
			for(int j=0; j<inLabels.length;j++){
				outLabels.add(inLabels[j]);
			}
		}
		
//*debug*/ System.err.println("labels ready");		
		
		//create outputFiles and writers
		try{
//			output data has same order as ref run
			outputWriters = new Hashtable<String,BufferedWriter>();
			if(outputFileDir == null){
				outputFileDir = inputDir.concat("recombinedRuns/");
				if(! (new File(outputFileDir)).mkdir()){
						System.err.println("can't create output directory!");
				}
				Iterator<String> it = outLabels.iterator();
				while(it.hasNext()){
					String label = it.next();
					File f = new File(outputFileDir.concat(label)+".rec");
					if(! f.createNewFile()){
						System.err.println("can't create file "+outputFileDir.concat(label));
					}
					FileWriter fw = new FileWriter(f);
					BufferedWriter bw = new BufferedWriter(fw);
					//write subDirNames as headings
					bw.write("year, ");
					for(int d=0; d<inputSubDirs.length; d++){
						bw.write(inputSubDirs[d]+", ");
					}
					bw.newLine();
					bw.flush();
					outputWriters.put(label,bw);
				}
			}
		}catch(Exception e){
			System.err.println(e);
		}
		
		
		////////
		//work on data
		
		
		for(int y=minYear; y<maxYear; y++){
			//read data lines
			inputDataLine = new Hashtable<String, double[]>();
			for(int i=0; i<inputSubDirs.length;i++){
				inputDataLine.put(inputSubDirs[i],readDataLine(inputReaders.get(inputSubDirs[i])));
			}
			

			String d;
			String[] inLabels; 
			double[] data;
			BufferedWriter w;
			
			try{
			
			//write one file at a time
			Iterator<String> itLabel = outLabels.iterator();
			while(itLabel.hasNext()){	
				String label = itLabel.next();
				w= outputWriters.get(label);
				
//				write year
				w.write(y+", ");
				
				//go through subDirs in order as represented in output 
				for(int i=0; i<inputSubDirs.length; i++){
					d= inputSubDirs[i];
					inLabels = inputLabels.get(d);
					data = inputDataLine.get(d);
					//lookup correct data entry
					for(int k=0;k<inLabels.length;k++){
						if(label.equals(inLabels[k])){ //found correct
							w.write(Double.toString(data[k]));
						}								
					}
					w.write(", ");
				}
				w.newLine();
				w.flush();
					
			}
				
			}catch(Exception e){};

		}

	}
	//for each subDir which contains the dataSet a reader for subDir/refDataSet is created
	protected Hashtable<String,BufferedReader> createInputReaders(){
	
		
		BufferedReader r;
		Hashtable<String,BufferedReader> readers = new Hashtable<String,BufferedReader>();
		//if no inputSubDirs are specified use all subDirs in mainDir
		//there may be no other subDirs in mainDir!!
		if(inputSubDirs == null){
			//create array of all Files in inputMainDir which are directories
			File[] sDirFiles = (new File(inputDir)).listFiles(new FileFilter(){
				public boolean accept(File pathname){
					return pathname.isDirectory();
				}
			});
			
			ArrayList<String> dummy = new ArrayList<String>();
					
			
			for(int i=0; i<sDirFiles.length; i++){
				String sDirName = sDirFiles[i].getName()+"/";
				r = createReader(inputDir.concat(sDirName).concat(dataSetName));
				if(r!= null) {
					readers.put(sDirName, r);
					dummy.add(sDirName); 
				}
			}
			inputSubDirs = dummy.toArray(new String[dummy.size()]);
		}
		return readers;
	}
	
	protected BufferedReader createReader(String absoluteFileName){
		
		File f = new File(absoluteFileName);
		BufferedReader r = null;
		try{
			r = new BufferedReader(new FileReader(f));
		}catch(Exception e){
			System.err.println(e);
			return null;
		}
		return r;
	}
	
	//reads labels for refRun
	protected String[] readLabels(BufferedReader refReader){
		
		String[] labels;
		
		StringTokenizer st;
		String line=null;
		
		//read intro (params etc.)
		do{
			try{
				line=refReader.readLine();
			}catch(Exception e){
				System.err.println(e);
			}
				if(line != null){
					if(line.startsWith("\"tick\"")){
						break;
					}
					if(line.length()==0){ //an empty line signals start of labels and data
						try{
							line=refReader.readLine();
						}catch(Exception e){
							System.err.println(e);
						}
						break;
					}		
				}
		}while(line!=null);	
		
		//create StringTokenizer for labels
		String delim = ",";
		st = new StringTokenizer(line,delim);
		String label = null;
//		st.nextToken(); //read away the label 'tick'	
		ArrayList<String> labelList = new ArrayList<String>();
		while(st.hasMoreTokens()){
			//remove '"' from labels
			label = st.nextToken().replace('"', ' ').trim();
			labelList.add(label);
		}
		//memorize labels in found order
		labels = labelList.toArray(new String[labelList.size()]);
		
		return labels;
	}
	
	protected double[] readDataLine(BufferedReader r){
		
		String delim = ",";
		StringTokenizer st = nextLine(r, delim);
		
		if(st == null){
			return null;
		}
		
		ArrayList<Double> dataList = new ArrayList<Double>();
		
		double d = nextDataToken(st);
		
		while(d>=0.0){
			dataList.add(new Double(d));
			d = nextDataToken(st);
		}
		Double dummy[] = dataList.toArray(new Double[dataList.size()]);	
		double data[] = new double[dummy.length];
		for(int i=0;i<dummy.length; i++){
			data[i] = dummy[i].doubleValue();
		}	
		return data;
		
	}
		
	public StringTokenizer nextLine(BufferedReader reader, String delim){
		
		String line =null;
		try{
			line=reader.readLine();
		}catch(Exception e){
			System.err.println(e);
			return null;
		}
		if(line == null){
			return null;
		}
		
		return new StringTokenizer(line,delim);
	}
	
	//returns -1.0 to signal end of line
	public double nextDataToken(StringTokenizer st){
		
		String token = null;
		if(st.hasMoreTokens()){
			try{
				token = st.nextToken();
			}catch(Exception e){
				System.err.println(e);
			}
			if(token.trim().length()==0){
				return 0.0;
			}
			else{
				return Double.valueOf(token);
			}
		}
		return -1.0;
	}
	
	/**value v in % of ref*/
	public double manipulateDataEntry(double ref, double v) throws Exception{
		if(ref==0.0){
			throw new Exception();
		}
		return v/ref*100;
	}

}
