package visualization;

import uchicago.src.sim.analysis.OpenSequenceGraph;
import uchicago.src.sim.analysis.Sequence;
import uchicago.src.sim.analysis.plot.OpenGraph;
import uchicago.src.sim.engine.SimModel;
import uchicago.src.sim.engine.SimpleModel;
import uchicago.src.sim.engine.SimModelImpl;

import java.io.*;
import java.util.StringTokenizer;

import model.Crop;
import model.IrrigationTechnology;

/** This class provides the possibility to visualize a model run whose data is stored in the
 * directory 'mainDir'. For each file in that dir a panel is launched which represents the respective
 * data.
 * @author gholtz
 *
 */


public class ShowModelOutput extends SimpleModel{

	public static String mainDir = 
		"C:/Georg/Model/Analyse/runs/_50/";	
	public static boolean multipleDirs = false;

	
protected class StringReaderSequence implements Sequence{
		
		DataPanel dp;
	
		public StringReaderSequence(DataPanel dp){
			this.dp = dp;
		}
		
		public double getSValue(){
			
			StringTokenizer st = dp.getStringTokenizer();
			String token = null;
	
			if(st.hasMoreTokens()){
				try{
					token = st.nextToken();
				}catch(Exception e){
					System.err.println(e);
				}
				if(token.trim().length()==0){
					return 0.0;
				}
				else{
					return Double.valueOf(token);
				}
			}
			return 0.0;
		}
	}

	
	
	protected class DataPanel extends OpenSequenceGraph{
		
		String dir, fileName;
		public BufferedReader reader;
		public StringTokenizer st;
		public String line;
		
		public DataPanel(String dir, String fileName, SimModel model){
			
			super(dir.concat("/"+fileName),model);	
			this.dir = dir;
			this.fileName = fileName;
		}
		
		public void launch(){	
			//create Buffered Reader to read lines from
			File f = new File(dir.concat(fileName));
			reader = null;
			
			try{
				reader = new BufferedReader(new FileReader(f));
			}catch(Exception e){
				System.err.println(e);
			}
			
			//read intro (params etc.)
			
			do{
				try{
					line=reader.readLine();
				}catch(Exception e){
					System.err.println(e);
				}
					if(line != null){
						//if(line.startsWith("\"tick\"")){
						if(line.length()==0){ //an empty line signals start of labels and data
							try{
								line=reader.readLine();
							}catch(Exception e){
								System.err.println(e);
							}
							break;
						}
					}
			}while(line!=null);	
			
			//create StringTokenizer for labels
			String delim = ",";
			st = new StringTokenizer(line,delim);

			String label;
			
			st.nextToken(); //read away the label 'tick'
			
			while(st.hasMoreTokens()){
				
				//remove '"' from labels
				label = st.nextToken().replace('"', ' ').trim();
				
				java.awt.Color color = null;
				int style = OpenGraph.PLUS_SIGN; 	//default
				boolean setStyle = false;
				
				//Adapt representation to style used in 'online'-simulations
				
				//get color
				if(label.startsWith("highYieldCereals") ||label.startsWith("High yield cereals") ){
					color = OptionRepresentation.colorForCropType(Crop.HIGHYIELDCEREALS);
					setStyle = true;
				}
				if(label.startsWith("tradCereals")|| label.startsWith("Traditional cereals")){
					color = OptionRepresentation.colorForCropType(Crop.TRADITIONALCEREALS);
					setStyle = true;
				}
				if(label.startsWith("sunflower") || label.startsWith("Sunflower")){
					color = OptionRepresentation.colorForCropType(Crop.SUNFLOWER);
					setStyle = true;
				}
				if(label.startsWith("sugarbeet")|| label.startsWith("Sugarbeet")){
					color = OptionRepresentation.colorForCropType(Crop.SUGARBEET);
					setStyle = true;
				}
				if(label.startsWith("vineyards") || label.startsWith("Vineyards")){
					color = OptionRepresentation.colorForCropType(Crop.VINEYARDS);
					setStyle = true;
				}
				if(label.startsWith("olives")|| label.startsWith("Olives")){
					color = OptionRepresentation.colorForCropType(Crop.OLIVES);
					setStyle = true;
				}
				if(label.startsWith("melons")|| label.startsWith("Melons")){
					color = OptionRepresentation.colorForCropType(Crop.MELONS);
					setStyle = true;
				}
				if(label.startsWith("garlic")|| label.startsWith("Garlic")){
					color = OptionRepresentation.colorForCropType(Crop.GARLIC);
					setStyle = true;
				}
				if(label.startsWith("paprica")|| label.startsWith("Paprica")){
					color = OptionRepresentation.colorForCropType(Crop.PAPRICA);
					setStyle = true;
				}
				if(label.startsWith("fallow")){
					color = OptionRepresentation.colorForCropType(Crop.FALLOW);
					setStyle = true;
				}
				
				//get markStyle
				if(label.endsWith("rainfed")){
					style = OptionRepresentation.markStyleForTechType(IrrigationTechnology.RAINFED);
					setStyle = true;
				}
				if(label.endsWith("drip")){
					style = OptionRepresentation.markStyleForTechType(IrrigationTechnology.DRIP);
					setStyle = true;
				}
				if(label.endsWith("flood")){
					style = OptionRepresentation.markStyleForTechType(IrrigationTechnology.FLOOD);
					setStyle = true;
				}
				if(label.endsWith("sprinkler")){
					style = OptionRepresentation.markStyleForTechType(IrrigationTechnology.SPRINKLER);
					setStyle = true;
				}
				
					if(setStyle){
						this.addSequence(label, new StringReaderSequence(this), color ,style );
					}
					else{
						this.addSequence(label, new StringReaderSequence(this));
					}
			}
			this.setXRange(1955, 2015);
			this.display();
		}
		
		/*read next dataLine and feed it into a new StringTokenizer
		 * which can be accessed by the sequences*/
		public void nextDataLine(){
			try{
				line=reader.readLine();
			}catch(Exception e){
				System.err.println(e);
			}
			
			st = new StringTokenizer(line,",");
		}
		
		//removes one token, used to remove the sequence 'tick' from data
		protected void eatTick(){
	
			try{ 
				st.nextToken();
			}catch(Exception e){
				System.err.println(e);
			}
		}
		
		public StringTokenizer getStringTokenizer(){
			return st;
		}
	}	
	
	
	////////////////////////////////////////////////////////////
	
	
	
	String dirName = null;
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		
		
		
		if(multipleDirs){
			File mDir = new File(mainDir);
			File [] sDirs = mDir.listFiles(new FileFilter(){
				public boolean accept(File pathname){
					return pathname.isDirectory();
				}
			});
			
			ShowModelOutput s;
			for(int i=0; i<sDirs.length; i++){
				s = new ShowModelOutput();
				s.dirName = mainDir.concat(sDirs[i].getName()).concat("/");
				s.run();
			}
		}
		else{
			new ShowModelOutput().run();
		}
			
	}
	
	public int tick = 1960;
	
	public void run(){
	
		//navigate through dir and launch one panel for each file found
//		File dir = new File(dirName);
		
		if(dirName == null){
			dirName = mainDir;
		}
		
		//use all files (not directories) in dirName whose name contains '.mr' as input
		//create array of all Files in dirName which are files
		File[] files = (new File(dirName)).listFiles(new FileFilter(){
			public boolean accept(File pathname){
				return (pathname.isFile() && pathname.getName().contains(".mr"));
			}
		});
		String[] fileNames = new String[files.length];
		for(int i=0; i<files.length; i++){
			fileNames[i] = files[i].getName();
		}
		
		DataPanel [] panels = new DataPanel[fileNames.length];
		
		for(int i=0; i<fileNames.length;i++){
			panels[i] = new DataPanel(dirName, fileNames[i],this);
			panels[i].launch();
		}
		
		for(tick=1960;tick<2010;tick++){
			for(int i=0; i<panels.length;i++){
				panels[i].nextDataLine();
				panels[i].eatTick();
				panels[i].step();
				panels[i].display();
			}
		}
	}
	

	
	public double getTickCount(){
		return tick;
	}

}
