package visualization;

import uchicago.src.sim.analysis.OpenSequenceGraph;
import uchicago.src.sim.analysis.Sequence;
import uchicago.src.sim.analysis.plot.OpenGraph;
import uchicago.src.sim.engine.SimModel;
import uchicago.src.sim.engine.SimpleModel;
import uchicago.src.sim.engine.SimModelImpl;

import java.io.*;
import java.util.StringTokenizer;
import java.awt.Color;


import model.Crop;
import model.IrrigationTechnology;

/** This class provides the possibility to visualize a model run whose data is stored in the
 * directory 'mainDir'. For each file in that dir a panel is launched which represents the respective
 * data.
 * @author gholtz
 *
 */


public class ShowNormalizedRuns extends SimpleModel{

	public static String mainDir = 
		"/home/student/m/mnebel/Simulationen/AlphaAlt/Average/";	
	public static String[] highlight = null; //{"S1"};//{"kappa3"};//{"kappa0"};//

	
protected class StringReaderSequence implements Sequence{
		
		DataPanel dp;
	
		public StringReaderSequence(DataPanel dp){
			this.dp = dp;
		}
		
		public double getSValue(){
			
			StringTokenizer st = dp.getStringTokenizer();
			String token = null;
	
			if(st.hasMoreTokens()){
				try{
					token = st.nextToken();
				}catch(Exception e){
					System.err.println(e);
				}
				if(token.trim().length()==0){
					return Double.NaN;
				}
				else{
					return Double.valueOf(token);
				}
			}
			return 0.0;
		}
	}

	
	
	protected class DataPanel extends OpenSequenceGraph{
		
		String dir, fileName;
		public BufferedReader reader;
		public StringTokenizer st;
		public String line;
		
		public DataPanel(String dir, String fileName, SimModel model){
			super(fileName,model);	
			this.dir = dir;
			this.fileName = fileName;
		}
		
		public void launch(){	
			//create Buffered Reader to read lines from
			File f = new File(mainDir.concat(fileName));
			reader = null;
			
			try{
				reader = new BufferedReader(new FileReader(f));
			}catch(Exception e){
				System.err.println(e);
			}
			
			//read intro (params etc.)
			
			do{
				try{
					line=reader.readLine();
				}catch(Exception e){
					System.err.println(e);
				}
					if(line != null){
						if(line.startsWith("year")){
							break;
						}
					}
			}while(line!=null);	
			
			//create StringTokenizer for labels
			String delim = ",";
			st = new StringTokenizer(line,delim);

			String label;
			
			st.nextToken(); //read away the label 'tick'
			
			while(st.hasMoreTokens()){
				
				//remove '/' from labels
				label = st.nextToken().replace('/', ' ').trim();
				
				if(highlight == null){
					this.addSequence(label, new StringReaderSequence(this));
				}
				else{
					boolean match = true;
					for(int i=0; i< highlight.length; i++){
						if(! label.contains(highlight[i])){
							match  = false;
							break;
						}
					}
					if(match){
					
						this.addSequence(label, new StringReaderSequence(this), Color.red);
					}
					else{
						this.addSequence(label, new StringReaderSequence(this), Color.gray);
					}
				}
				
			}
			this.setXRange(1955, 2015);
			this.setYRange(0.0, 200.0);
			this.setYAutoExpand(true);
			this.display();
		}
		
		/*read next dataLine and feed it into a new StringTokenizer
		 * which can be accessed by the sequences*/
		public void nextDataLine(){
			try{
				line=reader.readLine();
			}catch(Exception e){
				System.err.println(e);
			}
			
			st = new StringTokenizer(line,",");
		}
		
		//removes one token, used to remove the sequence 'tick' from data
		protected void eatTick(){
	
			try{ 
				st.nextToken();
			}catch(Exception e){
				System.err.println(e);
			}
		}
		
		public StringTokenizer getStringTokenizer(){
			return st;
		}
	}	
	
	
	////////////////////////////////////////////////////////////
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		
		new ShowNormalizedRuns().run();
			
	}
	
	public int tick = 1960;
	
	public void run(){
	
		//navigate through mainDir and launch one panel for each file found
		File dir = new File(mainDir);
		String[] fileNames = dir.list();
		
		DataPanel [] panels = new DataPanel[fileNames.length];
		
		for(int i=0; i<fileNames.length;i++){
			panels[i] = new DataPanel(mainDir, fileNames[i],this);
			panels[i].launch();
		}
		
		for(tick=1960;tick<=2010;tick++){
			for(int i=0; i<panels.length;i++){
				panels[i].nextDataLine();
				panels[i].eatTick();
				panels[i].step();
				panels[i].display();
			}
		}
	}
	

	
	public double getTickCount(){
		return tick;
	}

}
