package visualization;

import java.util.Hashtable;
import java.util.ArrayList;

import uchicago.src.sim.analysis.OpenSequenceGraph;
import uchicago.src.sim.analysis.Sequence;
import uchicago.src.sim.engine.SimModel;

import model.Farmer;
import model.GlobalVariables;
import model.LandUse;
import model.LandUsePattern;

public class SingleFarmerLandUsePatternChart extends OpenSequenceGraph implements ObserveFarmerChart{

	protected int nrFarmer;
	protected Farmer toObserve;
	protected Hashtable<String,LandUseSequence> sequences;
	
	/** farmerToObserve gives nr of Farmer to Observe in List provided in setFarmerToObserve.
	 * This approach allows to implement the Interface ObserveFarmerChart.
	 */
	public SingleFarmerLandUsePatternChart(String title, SimModel model, int farmerToObserve){
		super(title, model, GlobalVariables.getOutputDir()+"SingleFarmerLandUsePattern_"+title+GlobalVariables.dataFileExtension,uchicago.src.sim.analysis.PlotModel.CSV);
		sequences = new Hashtable<String,LandUseSequence>();
		nrFarmer = farmerToObserve;
	}
	
	public void setFarmersToObserve(ArrayList<Farmer> f){
		toObserve = f.get(nrFarmer);
	}
	
	public void step(){
		
		if(toObserve != null){
			LandUsePattern p = toObserve.landUsePattern();
			LandUse l;
			LandUseSequence s;
			for(int i=0; i<p.length(); i++){
				l = p.get(i);
				if(! sequences.containsKey(l.getName())){ //use name of LandUse since objects are cloned from time to time
					s= new LandUseSequence(l);
					sequences.put(l.getName(), s);
					this.addSequence(l.getName(), s, OptionRepresentation.colorForOption(l.option),OptionRepresentation.markStyleForOption(l.option));
					this.display();
				}
/*				else{
					s = sequences.get(l.getName());
				}
*/			}
		}
		super.step();

	}
	
	protected class LandUseSequence implements Sequence {
		
		protected LandUse landUse;
		
		public LandUseSequence(LandUse l){
			landUse = l;
		}
		
		public double getSValue() {
			
			LandUsePattern p = toObserve.landUsePattern();
			LandUse l;
			double size =0;
			for(int i=0; i<p.length(); i++){
				l = p.get(i);
				if(l.equalOption(landUse)){
					size+=l.size;
				}
			}
			return size;
		}
	}
}
