package visualization;

import java.util.ArrayList;

import uchicago.src.sim.analysis.OpenSequenceGraph;
import uchicago.src.sim.analysis.Sequence;
import uchicago.src.sim.engine.SimModel;

import model.Farmer;
import model.GlobalVariables;
import model.decisionMaking.CompareLandUsePattern_Utility;

public class SingleFarmerScoreChart extends OpenSequenceGraph implements ObserveFarmerChart{

	public static final int GROSSMARGIN = 0;	
	public static final int RISK = 1;
	public static final int LEGAL = 2;		
	public static final int LABOUR = 3;		
		

	
	
	protected int nrFarmer;
	protected Farmer toObserve;
	public double refGM = 2000; //factor to downscale grossMargin in output
	
	/** farmerToObserve gives nr of Farmer to Observe in List provided in setFarmerToObserve.
	 * This approach allows to implement the Interface ObserveFarmerChart.
	 */
	public SingleFarmerScoreChart(String title, SimModel model, int farmerToObserve){
		super(title, model, GlobalVariables.getOutputDir()+"SingleFarmerScore_"+title+GlobalVariables.dataFileExtension,uchicago.src.sim.analysis.PlotModel.CSV);
		nrFarmer = farmerToObserve;
	}
	
	public void setFarmersToObserve(ArrayList<Farmer> f){
		toObserve = f.get(nrFarmer);
		this.addSequence("Rel. gross Margin", new ScoreSequence(GROSSMARGIN));
		this.addSequence("Risk", new ScoreSequence(RISK));
		this.addSequence("Labour", new ScoreSequence(LABOUR));
		this.addSequence("Stay legal", new ScoreSequence(LEGAL));
		this.addSequence("Total weighted score", new ScoreSequence(-1));
	}
	

	/**Works only for CompareLandUsePattern_Utility*/
	protected class ScoreSequence implements Sequence {
		
		
		int p;
		
		public ScoreSequence(int priority){
			p = priority;
		}
		
		public double getSValue() {
		
			return getScore(toObserve,p);
		}
		protected double getScore(Farmer f, int priority){
			
			CompareLandUsePattern_Utility cp = (CompareLandUsePattern_Utility)(f.choiceProcess.getComparePattern());
			if(priority ==GROSSMARGIN){
				return cp.evaluateGrossMargin(f.elp, f)/refGM;				
			}
			if(priority == LABOUR){
				return cp.evaluateLabour(f.elp, f);
			}
			if(priority == RISK){
				return cp.evaluateRisk(f.elp, f);
			}
			if(priority == LEGAL){
				return cp.evaluateLegality(f.elp, f);
			}
	/*		double s = cp.scoreGM(f.elp, f)*f.evaluationPreferences[Farmer.GROSS_MARGIN]*GlobalVariables.incomeFactor;
			s-= cp.scoreRisk(f.elp, f)*f.evaluationPreferences[Farmer.RISK]*GlobalVariables.riskFactor;
			s-=cp.scoreStayLegal(f.elp, f)*f.evaluationPreferences[Farmer.STAY_LEGAL]*GlobalVariables.legalFactor;
			s-= cp.scoreLabour(f.elp, f)*f.evaluationPreferences[Farmer.LABOUR]*GlobalVariables.labourFactor;
			return s;
	*/
			return cp.evaluatePattern(f.elp, f)/refGM;
		}	
	}
}
