package visualization;

import java.util.ArrayList;

import uchicago.src.sim.analysis.OpenSequenceGraph;
import uchicago.src.sim.analysis.Sequence;
import uchicago.src.sim.engine.SimModel;

import model.Farmer;
import model.GlobalVariables;
import model.rules.*;

public class SingleFarmerSubsidiesChart extends OpenSequenceGraph implements ObserveFarmerChart{

	protected int nrFarmer;
	protected Farmer toObserve;
	
	/** farmerToObserve gives nr of Farmer to Observe in List provided in setFarmerToObserve.
	 * This approach allows to implement the Interface ObserveFarmerChart.
	 */
	public SingleFarmerSubsidiesChart(String title, SimModel model, int farmerToObserve){
		super(title, model,GlobalVariables.getOutputDir()+"SingleFarmerSubsidies_"+title+GlobalVariables.dataFileExtension,uchicago.src.sim.analysis.PlotModel.CSV);
		nrFarmer = farmerToObserve;
	}
	
	public void setFarmersToObserve(ArrayList<Farmer> f){
		toObserve = f.get(nrFarmer);
		this.addSequence("Cap direct payments", new CapDirectSequence());
		this.addSequence("AP 50", new AEPSequence(AEP.OPTION50));
		this.addSequence("AP 70", new AEPSequence(AEP.OPTION70));
		this.addSequence("AP 100", new AEPSequence(AEP.OPTION100));
	}
	
	protected class CapDirectSequence implements Sequence {
		
		public double getSValue() {
		
			return toObserve.evaluatedLandUsePattern().capCompPayments;
		}
	}
	protected class AEPSequence implements Sequence {
		
		protected int o;
		
		public AEPSequence(int option){
			this.o = option;
		}
		
		public double getSValue() {
		
			if(toObserve.evaluatedLandUsePattern().aepOption == this.o){
				return toObserve.evaluatedLandUsePattern().aepCompPayments;
			}
			return 0.0;
		}
	}
}
