package visualization;

import java.util.ArrayList;

import model.Farmer;
import model.Farmer_Satisficing;
import model.GlobalVariables;
import uchicago.src.sim.analysis.OpenSequenceGraph;
import uchicago.src.sim.analysis.Sequence;
import uchicago.src.sim.analysis.plot.OpenGraph;
import uchicago.src.sim.engine.SimModel;

public class SingleFarmerThresholdOutputChart extends OpenSequenceGraph  implements ObserveFarmerChart{

	
	protected int nrFarmer;
	protected Farmer_Satisficing toObserve;
	public double refGM = 20000; //factor to downscale grossMargin in output
	
	
	public SingleFarmerThresholdOutputChart(String title, SimModel model, int farmerToObserve) {
		super(title, model, GlobalVariables.getOutputDir()+"SingleFarmerThresholdOutput_"+title+GlobalVariables.dataFileExtension,uchicago.src.sim.analysis.PlotModel.CSV);
		nrFarmer = farmerToObserve;
	}

	
	public void setFarmersToObserve(ArrayList<Farmer> f) {
		toObserve = (Farmer_Satisficing)f.get(nrFarmer);
		if (Farmer_Satisficing.profitIncluded){
			this.addSequence("Output profit", new ScoreSequence(Farmer_Satisficing.PROFIT, true), OptionRepresentation.colorForOutput_Threshold(Farmer_Satisficing.PROFIT, true), OpenGraph.FILLED_SQUARE);
			this.addSequence("Threshold profit", new ScoreSequence(Farmer_Satisficing.PROFIT, false), OptionRepresentation.colorForOutput_Threshold(Farmer_Satisficing.PROFIT, false), OpenGraph.CROSS);
		}
		
		if (Farmer_Satisficing.labourIncluded){
			this.addSequence("Output labour", new ScoreSequence(Farmer_Satisficing.LABOUR, true), OptionRepresentation.colorForOutput_Threshold(Farmer_Satisficing.LABOUR, true), OpenGraph.FILLED_SQUARE);
			this.addSequence("Threshold labour", new ScoreSequence(Farmer_Satisficing.LABOUR, false), OptionRepresentation.colorForOutput_Threshold(Farmer_Satisficing.LABOUR, false), OpenGraph.CROSS);
		}
		
		if (Farmer_Satisficing.riskIncluded){
			this.addSequence("Output risk", new ScoreSequence(Farmer_Satisficing.RISK, true), OptionRepresentation.colorForOutput_Threshold(Farmer_Satisficing.RISK, true), OpenGraph.FILLED_SQUARE);
			this.addSequence("Threshold risk", new ScoreSequence(Farmer_Satisficing.RISK, false), OptionRepresentation.colorForOutput_Threshold(Farmer_Satisficing.RISK, false), OpenGraph.CROSS);
		}
		
		if (Farmer_Satisficing.legalityIncluded){
			this.addSequence("Output legality", new ScoreSequence(Farmer_Satisficing.LEGALITY, true), OptionRepresentation.colorForOutput_Threshold(Farmer_Satisficing.LEGALITY, true), OpenGraph.FILLED_SQUARE);
			this.addSequence("Threshold legality", new ScoreSequence(Farmer_Satisficing.LEGALITY, false), OptionRepresentation.colorForOutput_Threshold(Farmer_Satisficing.LEGALITY, false), OpenGraph.CROSS);
		}
	}

	
	
	/**Works only for CompareLandUsePattern_Satisficing*/
	protected class ScoreSequence implements Sequence {
		
		
		int p;
		boolean output;
		
		public ScoreSequence(int priority, boolean b){
			p = priority;
			output = b;
		}
		
		public double getSValue() {
		
			return getScore(toObserve,p, output);
		}
		protected double getScore(Farmer_Satisficing f, int priority, boolean output){
			
			//downscale Profit
			if(priority == Farmer_Satisficing.PROFIT && output){
				return f.getCurrentOutput()[priority] / refGM;				
			}
			if(priority == Farmer_Satisficing.PROFIT && !output){
				return f.getThreshold()[priority] / refGM;				
			}
			
			//The rest
			if(output){
				return f.getCurrentOutput()[priority];
			}
			else{
				return f.getThreshold()[priority];
			}
			
			
		}	
	}
}
