package visualization;

import java.util.ArrayList;

import uchicago.src.sim.analysis.OpenSequenceGraph;
import uchicago.src.sim.analysis.Sequence;
import uchicago.src.sim.engine.SimModel;

import model.Farmer;
import model.GlobalVariables;
import model.rules.*;

public class SingleFarmerWaterChart extends OpenSequenceGraph implements ObserveFarmerChart{

	protected int nrFarmer;
	protected Farmer toObserve;
	protected PumpingQuotas pq;
	
	/** farmerToObserve gives nr of Farmer to Observe in List provided in setFarmerToObserve.
	 * This approach allows to implement the Interface ObserveFarmerChart.
	 */
	public SingleFarmerWaterChart(String title, SimModel model, int farmerToObserve){
		super(title, model, GlobalVariables.getOutputDir()+"SingleFarmerWater_"+title+GlobalVariables.dataFileExtension,uchicago.src.sim.analysis.PlotModel.CSV);
		nrFarmer = farmerToObserve;
	}
	
	public void setPumpingQuotas(PumpingQuotas pq){
		this.pq = pq;
	}
	
	public void setFarmersToObserve(ArrayList<Farmer> f){
		toObserve = f.get(nrFarmer);
		this.addSequence("Water quota", new QuotaSequence());
		this.addSequence("Water used", new WaterUseSequence());
	}
	
	protected class QuotaSequence implements Sequence {
		
		public double getSValue() {
		
			if(pq!= null){
				try{
					return pq.waterQuota(toObserve, toObserve.evaluatedLandUsePattern());
				}catch(RuntimeException e){};
			}
			return Double.NaN;
		}
	}
	protected class WaterUseSequence implements Sequence {
		
		public double getSValue() {
		
			return toObserve.landUsePattern().waterUsed();
		}
	}
}
