package visualization;

import java.util.ArrayList;
import java.util.Iterator;

import uchicago.src.sim.analysis.OpenSequenceGraph;
import uchicago.src.sim.analysis.Sequence;
import uchicago.src.sim.analysis.plot.OpenGraph;
import uchicago.src.sim.engine.SimModel;

import model.Farmer;
import model.LandUse;
import model.Crop;
import model.Farming;
import model.LandUsePattern;
import model.GlobalVariables;

public class TotalCropAreaChart extends OpenSequenceGraph implements ObserveFarmerChart{

	protected ArrayList<Farmer> toObserve;
	protected double areas[];
	protected boolean cropsUsed[];
	protected boolean inited = false;
	
//	protected int[] types;
	
	public TotalCropAreaChart(String title, SimModel model){
		super(title, model, GlobalVariables.getOutputDir()+"TotalCropArea_"+title+GlobalVariables.dataFileExtension,uchicago.src.sim.analysis.PlotModel.CSV);
		cropsUsed = new boolean[11]; //all false
	//	this.setAxisTitles("time", "area (1000 ha)");
	}
	
	public void setFarmersToObserve(ArrayList<Farmer> f){
		toObserve = f;
	}
	
	public void step(){
		Farmer f;
		
/*		if(!inited){		
			this.addSequence("Traditional cereals", new CropAreaSequence(Crop.TRADITIONALCEREALS),OptionRepresentation.colorForCropType(Crop.TRADITIONALCEREALS),OpenGraph.CROSS);
			this.addSequence("Sunflower", new CropAreaSequence(Crop.SUNFLOWER),OptionRepresentation.colorForCropType(Crop.SUNFLOWER),OpenGraph.CROSS);
			this.addSequence("Vineyards", new CropAreaSequence(Crop.VINEYARDS),OptionRepresentation.colorForCropType(Crop.VINEYARDS),OpenGraph.CROSS);
			this.addSequence("Olives", new CropAreaSequence(Crop.OLIVES),OptionRepresentation.colorForCropType(Crop.OLIVES),OpenGraph.CROSS);
			this.addSequence("Fallow", new CropAreaSequence(Crop.FALLOW),OptionRepresentation.colorForCropType(Crop.FALLOW),OpenGraph.CROSS);
//			if(Farming.IRRIGATION){
				this.addSequence("High yield cereals", new CropAreaSequence(Crop.HIGHYIELDCEREALS),OptionRepresentation.colorForCropType(Crop.HIGHYIELDCEREALS),OpenGraph.CROSS);
				this.addSequence("Sugarbeet", new CropAreaSequence(Crop.SUGARBEET),OptionRepresentation.colorForCropType(Crop.SUGARBEET),OpenGraph.CROSS);
				this.addSequence("Potatoes", new CropAreaSequence(Crop.POTATOES),OptionRepresentation.colorForCropType(Crop.POTATOES),OpenGraph.CROSS);
				this.addSequence("Melons", new CropAreaSequence(Crop.MELONS),OptionRepresentation.colorForCropType(Crop.MELONS),OpenGraph.CROSS);
				this.addSequence("Paprica", new CropAreaSequence(Crop.PAPRICA),OptionRepresentation.colorForCropType(Crop.PAPRICA),OpenGraph.CROSS);
				this.addSequence("Garlic", new CropAreaSequence(Crop.GARLIC),OptionRepresentation.colorForCropType(Crop.GARLIC),OpenGraph.CROSS);		
//			}
			inited = true;
		}
*/			
		//sum up areas of crops
		if(toObserve != null){
			areas = new double[11];
			Iterator<Farmer> it = toObserve.iterator();
			double scalingFactor;
			while(it.hasNext()){
				f = it.next();
				scalingFactor = GlobalVariables.scalingFactor(f.sizeClass); 
				LandUsePattern p = f.landUsePattern();
				LandUse l;
				int t;
				for(int i=0; i<p.length(); i++){
					l = p.get(i);
					t = ((Farming)l.option).getCrop().getType();
					areas[t]+= l.size*scalingFactor;
				}
			}
			for(int i=0;i<cropsUsed.length; i++){
				//new crop found
				if((!cropsUsed[i]) && (areas[i]>0.0)){
					this.addSequence(Crop.getName(i), new CropAreaSequence(i),OptionRepresentation.colorForCropType(i),OpenGraph.CROSS);
					cropsUsed[i] = true;
				}
			}
			super.step();
			this.display();
		}
	}
	
	protected class CropAreaSequence implements Sequence {
		
		protected int type;
		
		public CropAreaSequence(int t){
			type = t;
		}
		//in 1000 ha
		public double getSValue() {
		
			return areas[type];
		}
	}
}
