package visualization;

import java.util.Hashtable;
import java.util.ArrayList;
import java.util.Iterator;

import uchicago.src.sim.analysis.OpenSequenceGraph;
import uchicago.src.sim.analysis.Sequence;
import uchicago.src.sim.engine.SimModel;

import model.Farmer;
import model.LandUse;
import model.LandUsePattern;
import model.GlobalVariables;

public class TotalLandUsePatternChart extends OpenSequenceGraph implements ObserveFarmerChart{

	protected ArrayList<Farmer> toObserve;
	protected Hashtable<String,LandUseSequence> sequences;
	
//	protected int[] types;
	
	public TotalLandUsePatternChart(String title, SimModel model){
		super(title, model, GlobalVariables.getOutputDir()+"TotalLandUsePattern_"+title+GlobalVariables.dataFileExtension,uchicago.src.sim.analysis.PlotModel.CSV);
		sequences = new Hashtable<String,LandUseSequence>();
	}
	
	public void setFarmersToObserve(ArrayList<Farmer> f){
		toObserve = f;
	}
	
	public void step(){
		if(toObserve != null){
			Farmer f;
			Iterator<Farmer> it = toObserve.iterator();
			while(it.hasNext()){
				f = it.next();
				LandUsePattern p = f.landUsePattern();
				LandUse l;
				LandUseSequence s;
				for(int i=0; i<p.length(); i++){
					l = p.get(i);
					if(! sequences.containsKey(l.getName())){ //use name of LandUse since objects are cloned from time to time
						s= new LandUseSequence(l);
						sequences.put(l.getName(), s);
						this.addSequence(l.getName(), s, OptionRepresentation.colorForOption(l.option),OptionRepresentation.markStyleForOption(l.option));
						this.display();
					}
				}
			}
		}
		super.step();

	}
	
	protected class LandUseSequence implements Sequence {
		
		protected LandUse landUse;
		
		public LandUseSequence(LandUse l){
			landUse = l;
		}
		
		public double getSValue() {
			
			Farmer f;
			Iterator<Farmer> it = toObserve.iterator();
			double size =0;
			double scalingFactor;
			while(it.hasNext()){
				f = it.next();
				scalingFactor = GlobalVariables.scalingFactor(f.sizeClass); 
				LandUsePattern p = f.landUsePattern();
				LandUse l;
				
				for(int i=0; i<p.length(); i++){
					l = p.get(i);
					if(l.equalOption(landUse)){
						size+=l.size*scalingFactor;
					}
				}
			}
			return size;
		}
	}
}
